# -*- coding: utf-8 -*-
"""Cullinan Codec Base Classes

定义 BodyCodec 和 ResponseCodec 抽象基类。

Author: Plumeink
"""

from abc import ABC, abstractmethod
from typing import Any, Dict, List, Optional


class BodyCodec(ABC):
    """请求体编解码器抽象基类

    职责:
    - decode: bytes → dict (请求解码)
    - encode: dict → bytes (可选，用于某些场景)

    特点:
    - 纯粹的编解码，不涉及业务逻辑
    - 可在 Middleware 中统一调用

    Example:
        class JsonBodyCodec(BodyCodec):
            content_types = ['application/json']

            def decode(self, body, charset='utf-8'):
                return json.loads(body.decode(charset))
    """

    # 支持的 Content-Type 列表 (子类必须重写)
    content_types: List[str] = []

    # 优先级 (数字越小优先级越高)
    priority: int = 100

    @abstractmethod
    def decode(self, body: bytes, charset: str = 'utf-8') -> Dict[str, Any]:
        """解码请求体

        Args:
            body: 原始请求体字节
            charset: 字符编码

        Returns:
            解码后的字典

        Raises:
            DecodeError: 解码失败
        """
        pass

    def encode(self, data: Dict[str, Any], charset: str = 'utf-8') -> bytes:
        """编码数据 (可选实现)

        Args:
            data: 要编码的字典
            charset: 字符编码

        Returns:
            编码后的字节

        Raises:
            EncodeError: 编码失败
        """
        raise NotImplementedError("This codec does not support encoding")

    @classmethod
    def supports(cls, content_type: str) -> bool:
        """检查是否支持该 Content-Type

        Args:
            content_type: HTTP Content-Type 头

        Returns:
            是否支持
        """
        if not content_type:
            return False
        ct_lower = content_type.lower()
        for ct in cls.content_types:
            if ct in ct_lower:
                return True
        return False


class ResponseCodec(ABC):
    """响应体编解码器抽象基类

    职责:
    - encode: Any → bytes (响应编码)
    - 设置正确的 Content-Type

    使用场景:
    - 统一响应格式 (JSON/XML/MessagePack)
    - 内容协商 (Accept header)

    Example:
        class JsonResponseCodec(ResponseCodec):
            content_type = 'application/json'
            accept_types = ['application/json', '*/*']

            def encode(self, data, charset='utf-8'):
                return json.dumps(data).encode(charset)
    """

    # 输出的 Content-Type
    content_type: str = 'application/octet-stream'

    # 支持的 Accept 类型
    accept_types: List[str] = []

    # 优先级
    priority: int = 100

    @abstractmethod
    def encode(self, data: Any, charset: str = 'utf-8') -> bytes:
        """编码响应数据

        Args:
            data: 响应数据 (可以是任意类型)
            charset: 字符编码

        Returns:
            编码后的字节

        Raises:
            EncodeError: 编码失败
        """
        pass

    def get_content_type(self, charset: str = 'utf-8') -> str:
        """获取完整的 Content-Type 头

        Args:
            charset: 字符编码

        Returns:
            完整的 Content-Type 字符串
        """
        if 'charset' not in self.content_type.lower():
            return f"{self.content_type}; charset={charset}"
        return self.content_type

    @classmethod
    def supports_accept(cls, accept: str) -> bool:
        """检查是否支持该 Accept 类型

        Args:
            accept: HTTP Accept 头

        Returns:
            是否支持
        """
        if not accept or accept == '*/*':
            return True
        accept_lower = accept.lower()
        for at in cls.accept_types:
            if at in accept_lower:
                return True
        return False




# -*- coding: utf-8 -*-
"""Cullinan ParamValidator Tests

测试参数校验器。

Author: Plumeink
"""

import unittest

from cullinan.params import (
    ParamValidator,
    ValidationError,
    Param,
    Query,
)


class TestValidationError(unittest.TestCase):
    """测试 ValidationError"""

    def test_basic_error(self):
        """基本错误"""
        err = ValidationError("test error", param_name="age", value=5)
        self.assertEqual(err.message, "test error")
        self.assertEqual(err.param_name, "age")
        self.assertEqual(err.value, 5)

    def test_to_dict(self):
        """转换为字典"""
        err = ValidationError("test", param_name="x", value=1, constraint="ge:0")
        d = err.to_dict()
        self.assertEqual(d['message'], "test")
        self.assertEqual(d['param'], "x")
        self.assertEqual(d['constraint'], "ge:0")


class TestParamValidatorRequired(unittest.TestCase):
    """测试必填校验"""

    def test_required_with_value(self):
        """必填有值"""
        ParamValidator.validate_required("test", True, "name")  # 不抛出

    def test_required_without_value(self):
        """必填无值"""
        with self.assertRaises(ValidationError) as ctx:
            ParamValidator.validate_required(None, True, "name")
        self.assertIn("required", ctx.exception.message)

    def test_optional_without_value(self):
        """可选无值"""
        ParamValidator.validate_required(None, False, "name")  # 不抛出


class TestParamValidatorNumeric(unittest.TestCase):
    """测试数值校验"""

    def test_ge_pass(self):
        """ge 通过"""
        ParamValidator.validate_ge(10, 5, "age")
        ParamValidator.validate_ge(5, 5, "age")

    def test_ge_fail(self):
        """ge 失败"""
        with self.assertRaises(ValidationError):
            ParamValidator.validate_ge(3, 5, "age")

    def test_ge_none(self):
        """ge None 跳过"""
        ParamValidator.validate_ge(None, 5, "age")

    def test_le_pass(self):
        """le 通过"""
        ParamValidator.validate_le(5, 10, "age")
        ParamValidator.validate_le(10, 10, "age")

    def test_le_fail(self):
        """le 失败"""
        with self.assertRaises(ValidationError):
            ParamValidator.validate_le(15, 10, "age")

    def test_gt_pass(self):
        """gt 通过"""
        ParamValidator.validate_gt(10, 5, "age")

    def test_gt_fail(self):
        """gt 失败 (等于也不行)"""
        with self.assertRaises(ValidationError):
            ParamValidator.validate_gt(5, 5, "age")

    def test_lt_pass(self):
        """lt 通过"""
        ParamValidator.validate_lt(5, 10, "age")

    def test_lt_fail(self):
        """lt 失败 (等于也不行)"""
        with self.assertRaises(ValidationError):
            ParamValidator.validate_lt(10, 10, "age")


class TestParamValidatorLength(unittest.TestCase):
    """测试长度校验"""

    def test_min_length_pass(self):
        """min_length 通过"""
        ParamValidator.validate_min_length("hello", 3, "name")
        ParamValidator.validate_min_length([1, 2, 3], 2, "items")

    def test_min_length_fail(self):
        """min_length 失败"""
        with self.assertRaises(ValidationError):
            ParamValidator.validate_min_length("hi", 3, "name")

    def test_max_length_pass(self):
        """max_length 通过"""
        ParamValidator.validate_max_length("hi", 10, "name")

    def test_max_length_fail(self):
        """max_length 失败"""
        with self.assertRaises(ValidationError):
            ParamValidator.validate_max_length("hello world", 5, "name")


class TestParamValidatorRegex(unittest.TestCase):
    """测试正则校验"""

    def test_regex_pass(self):
        """regex 通过"""
        ParamValidator.validate_regex("test@example.com", r".+@.+\..+", "email")

    def test_regex_fail(self):
        """regex 失败"""
        with self.assertRaises(ValidationError):
            ParamValidator.validate_regex("invalid-email", r".+@.+\..+", "email")

    def test_regex_none(self):
        """regex None 跳过"""
        ParamValidator.validate_regex(None, r".+", "field")


class TestParamValidatorBatch(unittest.TestCase):
    """测试批量校验"""

    def test_validate_multiple(self):
        """多个规则"""
        validators = [('ge', 0), ('le', 100)]
        ParamValidator.validate(50, validators, "score")

    def test_validate_fail_first(self):
        """第一个规则失败"""
        validators = [('ge', 10), ('le', 100)]
        with self.assertRaises(ValidationError):
            ParamValidator.validate(5, validators, "score")

    def test_validate_unknown_rule(self):
        """未知规则"""
        with self.assertRaises(ValueError):
            ParamValidator.validate(10, [('unknown', 5)], "x")


class TestParamValidatorWithParam(unittest.TestCase):
    """测试与 Param 类集成"""

    def test_validate_param_required(self):
        """校验 Param 必填"""
        param = Query(int, name="age")
        with self.assertRaises(ValidationError):
            ParamValidator.validate_param(param, None)

    def test_validate_param_with_constraints(self):
        """校验 Param 约束"""
        param = Query(int, name="age", ge=0, le=150)
        ParamValidator.validate_param(param, 25)

        with self.assertRaises(ValidationError):
            ParamValidator.validate_param(param, -5)

    def test_validate_param_optional(self):
        """校验可选 Param"""
        param = Query(int, name="page", default=1)
        ParamValidator.validate_param(param, None)  # 不抛出，因为有默认值 = 不必填


if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-
"""Cullinan Pydantic Model Handler

可选的 Pydantic 模型处理器。

Pydantic 是可选依赖，此模块仅在 Pydantic 可用时才能正常工作。

Author: Plumeink
"""

from typing import Any, Dict, Type

from .base import ModelHandler, ModelHandlerError


def _is_pydantic_available() -> bool:
    """检查 Pydantic 是否可用"""
    try:
        import pydantic
        return True
    except ImportError:
        return False


def _get_pydantic_version() -> int:
    """获取 Pydantic 主版本号"""
    try:
        import pydantic
        version_str = getattr(pydantic, 'VERSION', getattr(pydantic, '__version__', '1.0'))
        return int(version_str.split('.')[0])
    except Exception:
        return 0


class PydanticHandler(ModelHandler):
    """Pydantic 模型处理器

    可选处理器，用于解析 Pydantic BaseModel。

    Features:
    - 支持 Pydantic v1 和 v2
    - 完整的 Pydantic 校验
    - 嵌套模型解析
    - 自动类型转换

    Note:
        此处理器仅在 Pydantic 已安装时可用。
    """

    priority = 50  # 高于 dataclass
    name = "pydantic"

    def __init__(self):
        if not _is_pydantic_available():
            raise ImportError("Pydantic is not installed")

        self._version = _get_pydantic_version()
        self._base_model = None
        self._load_pydantic()

    def _load_pydantic(self):
        """加载 Pydantic"""
        from pydantic import BaseModel
        self._base_model = BaseModel

    def can_handle(self, type_: Type) -> bool:
        """检查是否是 Pydantic BaseModel"""
        if type_ is None:
            return False

        try:
            return isinstance(type_, type) and issubclass(type_, self._base_model)
        except Exception:
            return False

    def resolve(self, model_class: Type, data: Dict[str, Any]) -> Any:
        """解析数据为 Pydantic 模型实例"""
        if not self.can_handle(model_class):
            raise ModelHandlerError(
                f"{model_class} is not a Pydantic BaseModel",
                model_class=model_class,
                handler_name=self.name,
            )

        if data is None:
            data = {}

        try:
            if self._version >= 2:
                return model_class.model_validate(data)
            else:
                return model_class.parse_obj(data)
        except Exception as e:
            error_type = type(e).__name__
            if 'ValidationError' in error_type:
                raise ModelHandlerError(
                    f"Validation failed for {model_class.__name__}",
                    model_class=model_class,
                    errors=self._extract_errors(e),
                    handler_name=self.name,
                )
            raise ModelHandlerError(
                f"Failed to resolve {model_class.__name__}: {e}",
                model_class=model_class,
                handler_name=self.name,
            )

    def to_dict(self, instance: Any) -> Dict[str, Any]:
        """将 Pydantic 模型实例转换为字典"""
        if self._version >= 2:
            return instance.model_dump()
        else:
            return instance.dict()

    def to_json(self, instance: Any) -> str:
        """将 Pydantic 模型实例转换为 JSON 字符串"""
        if self._version >= 2:
            return instance.model_dump_json()
        else:
            return instance.json()

    def get_schema(self, model_class: Type) -> Dict[str, Any]:
        """获取 Pydantic 模型的 JSON Schema"""
        if not self.can_handle(model_class):
            raise ModelHandlerError(
                f"{model_class} is not a Pydantic BaseModel",
                model_class=model_class,
                handler_name=self.name,
            )

        if self._version >= 2:
            return model_class.model_json_schema()
        else:
            return model_class.schema()

    def _extract_errors(self, error) -> list:
        """从 Pydantic ValidationError 提取错误信息"""
        errors = []
        try:
            for err in error.errors():
                errors.append({
                    'loc': list(err.get('loc', [])),
                    'msg': err.get('msg', ''),
                    'type': err.get('type', ''),
                })
        except Exception:
            errors.append({'msg': str(error)})
        return errors




# -*- coding: utf-8 -*-
"""Cullinan Type Converter

类型转换器，将请求参数转换为目标类型。

Author: Plumeink
"""

import json
from typing import Any, Type, Union


class ConversionError(Exception):
    """类型转换错误

    Attributes:
        message: 错误消息
        value: 原始值
        target_type: 目标类型
    """

    def __init__(
        self,
        message: str,
        value: Any = None,
        target_type: Type = None
    ):
        super().__init__(message)
        self.message = message
        self.value = value
        self.target_type = target_type

    def __repr__(self) -> str:
        return f"ConversionError({self.message!r}, value={self.value!r}, target_type={self.target_type})"


class TypeConverter:
    """类型转换器

    将请求中的原始值转换为目标类型。

    Example:
        converter = TypeConverter()

        # 基本转换
        result = converter.convert("123", int)  # -> 123
        result = converter.convert("true", bool)  # -> True
        result = converter.convert("1,2,3", list)  # -> ["1", "2", "3"]
    """

    # 布尔值真值字符串
    TRUE_VALUES = frozenset({
        'true', 'True', 'TRUE',
        '1',
        'yes', 'Yes', 'YES',
        'on', 'On', 'ON'
    })

    FALSE_VALUES = frozenset({
        'false', 'False', 'FALSE',
        '0', '',
        'no', 'No', 'NO',
        'off', 'Off', 'OFF'
    })

    @classmethod
    def convert(cls, value: Any, target_type: Type) -> Any:
        """将值转换为目标类型

        Args:
            value: 原始值
            target_type: 目标类型

        Returns:
            转换后的值

        Raises:
            ConversionError: 转换失败
        """
        if value is None:
            return None

        # 如果已经是目标类型，直接返回
        if isinstance(value, target_type):
            return value

        try:
            if target_type is str:
                return cls._to_str(value)
            elif target_type is int:
                return cls._to_int(value)
            elif target_type is float:
                return cls._to_float(value)
            elif target_type is bool:
                return cls._to_bool(value)
            elif target_type is list:
                return cls._to_list(value)
            elif target_type is dict:
                return cls._to_dict(value)
            elif target_type is bytes:
                return cls._to_bytes(value)
            else:
                # 尝试直接调用目标类型构造函数
                return target_type(value)
        except ConversionError:
            raise
        except Exception as e:
            raise ConversionError(
                f"Cannot convert {type(value).__name__} to {target_type.__name__}: {e}",
                value=value,
                target_type=target_type
            )

    @classmethod
    def _to_str(cls, value: Any) -> str:
        """转换为字符串"""
        if isinstance(value, bytes):
            return value.decode('utf-8')
        return str(value)

    @classmethod
    def _to_int(cls, value: Any) -> int:
        """转换为整数"""
        if isinstance(value, str):
            value = value.strip()
            # 处理浮点数字符串
            if '.' in value:
                return int(float(value))
            # 处理空字符串
            if not value:
                raise ConversionError(
                    "Cannot convert empty string to int",
                    value=value,
                    target_type=int
                )
            return int(value)
        if isinstance(value, float):
            return int(value)
        if isinstance(value, bool):
            return 1 if value else 0
        return int(value)

    @classmethod
    def _to_float(cls, value: Any) -> float:
        """转换为浮点数"""
        if isinstance(value, str):
            value = value.strip()
            if not value:
                raise ConversionError(
                    "Cannot convert empty string to float",
                    value=value,
                    target_type=float
                )
        return float(value)

    @classmethod
    def _to_bool(cls, value: Any) -> bool:
        """转换为布尔值"""
        if isinstance(value, bool):
            return value
        if isinstance(value, str):
            if value in cls.TRUE_VALUES:
                return True
            if value in cls.FALSE_VALUES:
                return False
            raise ConversionError(
                f"Cannot convert '{value}' to bool",
                value=value,
                target_type=bool
            )
        if isinstance(value, (int, float)):
            return bool(value)
        return bool(value)

    @classmethod
    def _to_list(cls, value: Any) -> list:
        """转换为列表"""
        if isinstance(value, list):
            return value
        if isinstance(value, (tuple, set, frozenset)):
            return list(value)
        if isinstance(value, str):
            # 尝试解析 JSON 数组
            value = value.strip()
            if value.startswith('[') and value.endswith(']'):
                try:
                    result = json.loads(value)
                    if isinstance(result, list):
                        return result
                except json.JSONDecodeError:
                    pass
            # 逗号分隔
            if ',' in value:
                return [v.strip() for v in value.split(',')]
            # 单值
            if value:
                return [value]
            return []
        return [value]

    @classmethod
    def _to_dict(cls, value: Any) -> dict:
        """转换为字典"""
        if isinstance(value, dict):
            return value
        if isinstance(value, str):
            value = value.strip()
            if value.startswith('{') and value.endswith('}'):
                try:
                    result = json.loads(value)
                    if isinstance(result, dict):
                        return result
                except json.JSONDecodeError:
                    pass
            raise ConversionError(
                f"Cannot parse '{value}' as JSON dict",
                value=value,
                target_type=dict
            )
        raise ConversionError(
            f"Cannot convert {type(value).__name__} to dict",
            value=value,
            target_type=dict
        )

    @classmethod
    def _to_bytes(cls, value: Any) -> bytes:
        """转换为字节串"""
        if isinstance(value, bytes):
            return value
        if isinstance(value, str):
            return value.encode('utf-8')
        if isinstance(value, (bytearray, memoryview)):
            return bytes(value)
        raise ConversionError(
            f"Cannot convert {type(value).__name__} to bytes",
            value=value,
            target_type=bytes
        )

    @classmethod
    def can_convert(cls, value: Any, target_type: Type) -> bool:
        """检查是否可以转换

        Args:
            value: 原始值
            target_type: 目标类型

        Returns:
            是否可以转换
        """
        try:
            cls.convert(value, target_type)
            return True
        except (ConversionError, Exception):
            return False




# -*- coding: utf-8 -*-
"""测试 ControllerRegistry 依赖注入修复

Author: Plumeink
"""

import sys
sys.path.insert(0, '.')


def reset_all_registries():
    """重置所有注册表以便独立测试"""
    from cullinan.controller.registry import reset_controller_registry
    from cullinan.core.pending import PendingRegistry
    from cullinan.core import set_application_context

    reset_controller_registry()

    # 重置 PendingRegistry
    pending = PendingRegistry.get_instance()
    pending._registrations.clear()
    pending._frozen = False

    # 清除全局 ApplicationContext
    set_application_context(None)


def test_controller_di():
    """测试 Controller 依赖注入"""
    print("=" * 60)
    print("测试 1: 基础依赖注入")
    print("=" * 60)

    reset_all_registries()

    from cullinan.core import (
        ApplicationContext, 
        set_application_context, 
        get_application_context, 
        service, 
        controller, 
        Inject
    )

    # 创建一个测试服务
    @service
    class TestService:
        def get_message(self):
            return 'Hello from TestService'

    # 创建一个测试控制器
    @controller(url='/test')
    class TestController:
        test_service: TestService = Inject()
        
        def test_method(self):
            return self.test_service.get_message()

    # 创建 ApplicationContext 并设置全局引用
    ctx = ApplicationContext()
    set_application_context(ctx)
    ctx.refresh()

    print(f'[OK] ApplicationContext created and refreshed')
    print(f'[OK] Global context set: {get_application_context() is not None}')
    print(f'[OK] Is refreshed: {ctx.is_refreshed}')

    # 测试通过 ControllerRegistry 获取实例
    from cullinan.controller.registry import get_controller_registry
    controller_registry = get_controller_registry()

    # 注册控制器
    controller_registry.register('TestController', TestController, url_prefix='/test')

    # 获取实例
    instance = controller_registry.get_instance('TestController')
    print(f'[OK] Controller instance created: {instance}')
    print(f'[INFO] test_service type: {type(instance.test_service).__name__}')

    is_inject_object = isinstance(instance.test_service, Inject)
    print(f'[INFO] test_service is Inject object: {is_inject_object}')

    # 如果注入成功，调用方法
    if hasattr(instance.test_service, 'get_message'):
        message = instance.test_service.get_message()
        print(f'[OK] Message: {message}')
        print('[SUCCESS] Test 1 passed: Dependency injection works!')
        return True
    else:
        print('[FAILED] Test 1 failed: Dependency injection did not work')
        print(f'[DEBUG] instance.test_service = {instance.test_service}')
        print(f'[DEBUG] type = {type(instance.test_service)}')
        return False


def test_multiple_services():
    """测试多个服务注入"""
    print("\n" + "=" * 60)
    print("测试 2: 多服务依赖注入")
    print("=" * 60)

    reset_all_registries()

    from cullinan.core import (
        ApplicationContext,
        set_application_context,
        service,
        controller,
        Inject
    )

    @service
    class ServiceA:
        def name(self):
            return 'ServiceA'

    @service
    class ServiceB:
        def name(self):
            return 'ServiceB'

    @controller(url='/multi')
    class MultiServiceController:
        service_a: ServiceA = Inject()
        service_b: ServiceB = Inject()

        def get_names(self):
            return f'{self.service_a.name()} + {self.service_b.name()}'

    # 新建 ApplicationContext
    ctx = ApplicationContext()
    set_application_context(ctx)
    ctx.refresh()

    from cullinan.controller.registry import get_controller_registry
    controller_registry = get_controller_registry()
    controller_registry.register('MultiServiceController', MultiServiceController, url_prefix='/multi')

    instance = controller_registry.get_instance('MultiServiceController')

    # 检查所有服务是否正确注入
    a_ok = hasattr(instance.service_a, 'name')
    b_ok = hasattr(instance.service_b, 'name')

    print(f'[INFO] service_a injected correctly: {a_ok}')
    print(f'[INFO] service_b injected correctly: {b_ok}')

    if a_ok and b_ok:
        result = instance.get_names()
        print(f'[OK] Combined result: {result}')
        print('[SUCCESS] Test 2 passed: Multiple service injection works!')
        return True
    else:
        print('[FAILED] Test 2 failed')
        return False


def test_optional_injection():
    """测试可选依赖注入"""
    print("\n" + "=" * 60)
    print("测试 3: 可选依赖注入 (required=False)")
    print("=" * 60)

    reset_all_registries()

    from cullinan.core import (
        ApplicationContext,
        set_application_context,
        service,
        controller,
        Inject
    )

    @service
    class ExistingService:
        def exists(self):
            return True

    # 注意：MissingService 没有被 @service 装饰，所以不会被注册
    class MissingService:
        pass

    @controller(url='/optional')
    class OptionalController:
        existing: ExistingService = Inject()
        missing: MissingService = Inject(required=False)  # 可选依赖

    ctx = ApplicationContext()
    set_application_context(ctx)
    ctx.refresh()

    from cullinan.controller.registry import get_controller_registry
    controller_registry = get_controller_registry()
    controller_registry.register('OptionalController', OptionalController, url_prefix='/optional')

    instance = controller_registry.get_instance('OptionalController')

    existing_ok = hasattr(instance.existing, 'exists')
    missing_is_none = instance.missing is None

    print(f'[INFO] existing service injected: {existing_ok}')
    print(f'[INFO] missing service is None: {missing_is_none}')

    if existing_ok and missing_is_none:
        print('[SUCCESS] Test 3 passed: Optional injection works!')
        return True
    else:
        print('[FAILED] Test 3 failed')
        return False


if __name__ == '__main__':
    results = []

    results.append(test_controller_di())
    results.append(test_multiple_services())
    results.append(test_optional_injection())

    print("\n" + "=" * 60)
    print("测试总结")
    print("=" * 60)
    passed = sum(results)
    total = len(results)
    print(f'通过: {passed}/{total}')

    sys.exit(0 if all(results) else 1)




# -*- coding: utf-8 -*-
"""Cullinan Dataclass Model Handler

内置的 dataclass 模型处理器。

Author: Plumeink
"""

import dataclasses
from typing import Any, Dict, Type, get_type_hints, Union

from .base import ModelHandler, ModelHandlerError


class DataclassHandler(ModelHandler):
    """Dataclass 模型处理器

    内置处理器，用于解析 Python dataclass。

    Features:
    - 自动类型转换
    - 支持嵌套 dataclass
    - 支持默认值
    - 支持 Optional 类型
    """

    priority = 10  # 低于第三方库，作为兜底
    name = "dataclass"

    def can_handle(self, type_: Type) -> bool:
        """检查是否是 dataclass"""
        if type_ is None:
            return False
        return dataclasses.is_dataclass(type_) and isinstance(type_, type)

    def resolve(self, model_class: Type, data: Dict[str, Any]) -> Any:
        """解析数据为 dataclass 实例"""
        if not self.can_handle(model_class):
            raise ModelHandlerError(
                f"{model_class} is not a dataclass",
                model_class=model_class,
                handler_name=self.name,
            )

        if data is None:
            data = {}

        # 获取字段信息
        fields = dataclasses.fields(model_class)
        type_hints = {}
        try:
            type_hints = get_type_hints(model_class)
        except Exception:
            pass

        # 构建参数
        kwargs = {}
        field_errors = []

        for field in fields:
            field_name = field.name
            field_type = type_hints.get(field_name, field.type)

            # 检查是否有值
            if field_name in data:
                raw_value = data[field_name]

                try:
                    # 解析嵌套 dataclass
                    if self.can_handle(field_type):
                        if isinstance(raw_value, dict):
                            kwargs[field_name] = self.resolve(field_type, raw_value)
                        elif isinstance(raw_value, field_type):
                            kwargs[field_name] = raw_value
                        else:
                            raise ModelHandlerError(
                                f"Cannot convert {type(raw_value).__name__} to {field_type.__name__}"
                            )
                    else:
                        # 处理 Optional 类型
                        actual_type = self._unwrap_optional(field_type)
                        if raw_value is None:
                            kwargs[field_name] = None
                        elif actual_type is not None:
                            kwargs[field_name] = self._convert_value(raw_value, actual_type)
                        else:
                            kwargs[field_name] = raw_value

                except ModelHandlerError as e:
                    field_errors.append({
                        'field': field_name,
                        'error': str(e),
                        'value': raw_value
                    })
                except Exception as e:
                    field_errors.append({
                        'field': field_name,
                        'error': str(e),
                        'value': raw_value
                    })
            else:
                # 没有提供值
                if field.default is not dataclasses.MISSING:
                    kwargs[field_name] = field.default
                elif field.default_factory is not dataclasses.MISSING:
                    kwargs[field_name] = field.default_factory()
                elif self._is_optional(field_type):
                    kwargs[field_name] = None
                else:
                    field_errors.append({
                        'field': field_name,
                        'error': f"Field '{field_name}' is required",
                        'value': None
                    })

        if field_errors:
            raise ModelHandlerError(
                f"Failed to resolve dataclass {model_class.__name__}",
                model_class=model_class,
                errors=field_errors,
                handler_name=self.name,
            )

        try:
            return model_class(**kwargs)
        except Exception as e:
            raise ModelHandlerError(
                f"Failed to create {model_class.__name__}: {e}",
                model_class=model_class,
                handler_name=self.name,
            )

    def to_dict(self, instance: Any) -> Dict[str, Any]:
        """将 dataclass 实例转换为字典"""
        if not dataclasses.is_dataclass(instance):
            raise ModelHandlerError(
                f"{type(instance)} is not a dataclass instance",
                handler_name=self.name,
            )

        result = {}
        for field in dataclasses.fields(instance):
            value = getattr(instance, field.name)
            if dataclasses.is_dataclass(value):
                result[field.name] = self.to_dict(value)
            else:
                result[field.name] = value
        return result

    def _is_optional(self, type_hint) -> bool:
        """检查类型是否是 Optional"""
        origin = getattr(type_hint, '__origin__', None)
        if origin is Union:
            args = getattr(type_hint, '__args__', ())
            return type(None) in args
        return False

    def _unwrap_optional(self, type_hint) -> Type:
        """从 Optional[X] 中提取 X"""
        origin = getattr(type_hint, '__origin__', None)
        if origin is Union:
            args = getattr(type_hint, '__args__', ())
            non_none = [a for a in args if a is not type(None)]
            if len(non_none) == 1:
                return non_none[0]
        if isinstance(type_hint, type):
            return type_hint
        return None

    def _convert_value(self, value: Any, target_type: Type) -> Any:
        """基本类型转换"""
        if value is None:
            return None
        if isinstance(value, target_type):
            return value

        try:
            if target_type is str:
                return str(value)
            elif target_type is int:
                return int(value)
            elif target_type is float:
                return float(value)
            elif target_type is bool:
                if isinstance(value, str):
                    return value.lower() in ('true', '1', 'yes', 'on')
                return bool(value)
            else:
                return target_type(value)
        except Exception:
            return value




# -*- coding: utf-8 -*-
"""Cullinan Parameter Base Classes

定义参数标记类的基础设施。

Author: Plumeink
"""

from typing import Any, Callable, List, Optional, Type, Union


class _UNSET:
    """哨兵类，表示未设置的值

    用于区分"未设置"和"设置为 None"的情况。
    使用单例模式确保全局唯一。
    """
    _instance = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __repr__(self) -> str:
        return '<UNSET>'

    def __bool__(self) -> bool:
        return False

    def __copy__(self):
        return self

    def __deepcopy__(self, memo):
        return self


# 全局 UNSET 单例
UNSET = _UNSET()


class Param:
    """参数标记基类

    所有参数类型 (Path, Query, Body, Header, File) 的基类。
    用于在函数签名中标记参数的来源和约束。

    Attributes:
        name: 参数名称 (默认使用函数参数名)
        type_: 目标类型
        required: 是否必填
        default: 默认值
        description: 参数描述
        alias: 别名 (从请求中读取时使用的键名)

    Validator Attributes:
        ge: 大于等于 (数值类型)
        le: 小于等于 (数值类型)
        gt: 大于 (数值类型)
        lt: 小于 (数值类型)
        min_length: 最小长度 (字符串/列表)
        max_length: 最大长度 (字符串/列表)
        regex: 正则表达式 (字符串)

    Example:
        # 作为类型注解使用
        @get_api(url="/users/{id}")
        async def get_user(self, id: Path(int), verbose: Query(bool, default=False)):
            pass

        # 直接实例化
        param = Param(int, name='age', required=True, ge=0, le=150)
    """

    __slots__ = (
        'name', 'type_', 'required', 'default', 'description', 'alias',
        'ge', 'le', 'gt', 'lt', 'min_length', 'max_length', 'regex',
        '_validators'
    )

    # 参数来源标识 (子类重写)
    _source: str = 'unknown'

    def __init__(
        self,
        type_: Type = str,
        *,
        name: str = None,
        required: bool = True,
        default: Any = UNSET,
        description: str = '',
        alias: str = None,
        # 数值约束
        ge: Union[int, float, None] = None,
        le: Union[int, float, None] = None,
        gt: Union[int, float, None] = None,
        lt: Union[int, float, None] = None,
        # 长度约束
        min_length: Optional[int] = None,
        max_length: Optional[int] = None,
        # 正则约束
        regex: Optional[str] = None,
    ):
        """初始化参数标记

        Args:
            type_: 目标类型 (默认 str)
            name: 参数名称 (默认使用函数参数名)
            required: 是否必填 (有默认值时自动为 False)
            default: 默认值
            description: 参数描述 (用于文档生成)
            alias: 别名 (从请求中读取时使用的键名)
            ge: 大于等于约束
            le: 小于等于约束
            gt: 大于约束
            lt: 小于约束
            min_length: 最小长度约束
            max_length: 最大长度约束
            regex: 正则表达式约束
        """
        self.name = name
        self.type_ = type_
        self.default = default
        self.description = description
        self.alias = alias

        # 如果有默认值，则不是必填
        if default is not UNSET:
            self.required = False
        else:
            self.required = required

        # 存储约束条件
        self.ge = ge
        self.le = le
        self.gt = gt
        self.lt = lt
        self.min_length = min_length
        self.max_length = max_length
        self.regex = regex

        # 延迟构建验证器
        self._validators = None

    @property
    def source(self) -> str:
        """返回参数来源标识"""
        return self._source

    def get_validators(self) -> List[tuple]:
        """获取验证器列表 (延迟构建)

        Returns:
            验证器列表，每项为 (validator_name, value) 元组
        """
        if self._validators is None:
            self._validators = self._build_validators()
        return self._validators

    def _build_validators(self) -> List[tuple]:
        """构建验证器列表"""
        validators = []

        if self.ge is not None:
            validators.append(('ge', self.ge))
        if self.le is not None:
            validators.append(('le', self.le))
        if self.gt is not None:
            validators.append(('gt', self.gt))
        if self.lt is not None:
            validators.append(('lt', self.lt))
        if self.min_length is not None:
            validators.append(('min_length', self.min_length))
        if self.max_length is not None:
            validators.append(('max_length', self.max_length))
        if self.regex is not None:
            validators.append(('regex', self.regex))

        return validators

    def has_default(self) -> bool:
        """检查是否有默认值"""
        return self.default is not UNSET

    def get_default(self) -> Any:
        """获取默认值

        Returns:
            默认值，如果未设置则返回 None
        """
        if self.default is UNSET:
            return None
        return self.default

    def __repr__(self) -> str:
        parts = [f"{self.__class__.__name__}({self.type_.__name__}"]
        if self.name:
            parts.append(f", name={self.name!r}")
        if not self.required:
            parts.append(", required=False")
        if self.default is not UNSET:
            parts.append(f", default={self.default!r}")
        if self.alias:
            parts.append(f", alias={self.alias!r}")
        parts.append(")")
        return "".join(parts)

    def __eq__(self, other) -> bool:
        if not isinstance(other, Param):
            return False
        return (
            self.name == other.name and
            self.type_ == other.type_ and
            self.required == other.required and
            self.default == other.default
        )

    def __hash__(self) -> int:
        return hash((self.name, self.type_, self.required))

    @classmethod
    def as_required(cls, type_: Type = str, **kwargs) -> 'Param':
        """创建必填参数的快捷方法

        Example:
            # 以下两种写法等价
            avatar: File = File.as_required(max_size=5*1024*1024)
            avatar: File = File(required=True, max_size=5*1024*1024)

        Args:
            type_: 目标类型 (默认 str)
            **kwargs: 其他参数配置

        Returns:
            设置了 required=True 的 Param 实例
        """
        kwargs['required'] = True
        return cls(type_, **kwargs)




# -*- coding: utf-8 -*-
"""Cullinan Parameter Types

定义各种参数来源类型：Path, Query, Body, Header, File。

Author: Plumeink
"""

from typing import Any, List, Optional, Type

from .base import Param, UNSET


class Path(Param):
    """URL 路径参数

    从 URL 路径中提取的参数，如 /users/{id} 中的 id。
    路径参数始终是必填的。

    Example:
        @get_api(url="/users/{id}")
        async def get_user(self, id: int = Path()):
            # id 已经转换为 int 类型
            pass

        @get_api(url="/users/{id}/posts/{post_id}")
        async def get_post(self, id: int = Path(), post_id: int = Path()):
            pass
    """
    _source = 'path'

    def __init__(
        self,
        type_: Type = str,
        *,
        name: str = None,
        description: str = '',
        alias: str = None,
        # 数值约束
        ge: Any = None,
        le: Any = None,
        gt: Any = None,
        lt: Any = None,
        # 长度约束
        min_length: int = None,
        max_length: int = None,
        # 正则约束
        regex: str = None,
    ):
        """初始化路径参数

        Note:
            路径参数始终是必填的，不支持默认值。
        """
        super().__init__(
            type_=type_,
            name=name,
            required=True,  # 路径参数始终必填
            default=UNSET,  # 不支持默认值
            description=description,
            alias=alias,
            ge=ge,
            le=le,
            gt=gt,
            lt=lt,
            min_length=min_length,
            max_length=max_length,
            regex=regex,
        )


class Query(Param):
    """查询参数

    从 URL 查询字符串中提取的参数，如 ?page=1&size=10。

    Example:
        @get_api(url="/users")
        async def list_users(
            self,
            page: int = Query(default=1, ge=1),
            size: int = Query(default=10, ge=1, le=100),
            q: str = Query(required=False),
        ):
            pass

        # 纯类型注解自动作为 Query（v0.90a5+）
        @get_api(url="/items")
        async def list_items(self, page: int = 1, size: int = 10):
            pass
    """
    _source = 'query'

    # 继承 Param 的 __init__，无需重写


class Body(Param):
    """请求体参数

    从请求体中提取的参数。传输格式 (JSON/Form) 由 Codec 层处理。

    Example:
        # 单个字段（新的统一语法）
        @post_api(url="/users")
        async def create_user(
            self,
            name: str = Body(required=True),
            age: int = Body(default=0, ge=0),
        ):
            pass

        # 使用 as_required() 快捷方法
        @post_api(url="/users")
        async def create_user(
            self,
            name: str = Body.as_required(min_length=1),
        ):
            pass

        # 配合 DynamicBody 使用整个请求体
        @post_api(url="/users")
        async def create_user(self, body: DynamicBody):
            print(body.name, body.age)
    """
    _source = 'body'

    # 继承 Param 的 __init__，无需重写


class RawBody(Param):
    """原始请求体 (未解析的 bytes)

    获取未解析的原始请求体 bytes，适用于：
    - 签名验证（如 GitHub Webhook）
    - 自定义解析格式
    - 二进制数据处理

    Example:
        @post_api(url="/webhook")
        async def handle_webhook(
            self,
            sign: str = Header(alias="X-Hub-Signature-256"),
            event: str = Header(alias="X-GitHub-Event"),
            raw_body: bytes = RawBody(),  # 推荐语法
        ):
            # raw_body 是 bytes 类型
            import hmac
            expected = hmac.new(secret, raw_body, 'sha256').hexdigest()
            if sign != f'sha256={expected}':
                raise ValueError('Invalid signature')

            # 手动解析
            import json
            data = json.loads(raw_body)
    """
    _source = 'raw_body'

    def __init__(
        self,
        *,
        name: str = None,
        required: bool = False,
        description: str = '',
    ):
        """初始化原始请求体参数

        Args:
            name: 参数名称
            required: 是否必填（默认 False）
            description: 参数描述
        """
        super().__init__(
            type_=bytes,
            name=name,
            required=required,
            default=b'',
            description=description,
        )


class Header(Param):
    """请求头参数

    从 HTTP 请求头中提取的参数。
    通常使用 alias 指定实际的请求头名称。
    HTTP 头匹配是大小写不敏感的（符合 RFC 7230）。

    Example:
        @get_api(url="/users")
        async def list_users(
            self,
            auth: str = Header(alias='Authorization', required=True),
            request_id: str = Header(alias='X-Request-ID', required=False),
        ):
            pass

        # GitHub Webhook 签名验证
        @post_api(url="/webhook")
        async def handle_webhook(
            self,
            sign: str = Header(alias="X-Hub-Signature-256"),
            event: str = Header(alias="X-GitHub-Event"),
            raw_body: RawBody,
        ):
            pass
    """
    _source = 'header'

    def __init__(
        self,
        type_: Type = str,
        *,
        name: str = None,
        required: bool = True,
        default: Any = UNSET,
        description: str = '',
        alias: str = None,
        # 长度约束
        min_length: int = None,
        max_length: int = None,
        # 正则约束
        regex: str = None,
    ):
        """初始化请求头参数

        Note:
            请求头参数通常使用 alias 指定实际的头名称，
            如 Authorization, Content-Type, X-Request-ID 等。
        """
        super().__init__(
            type_=type_,
            name=name,
            required=required,
            default=default,
            description=description,
            alias=alias,
            min_length=min_length,
            max_length=max_length,
            regex=regex,
        )


class File(Param):
    """文件参数

    从 multipart/form-data 请求中提取的文件。

    Attributes:
        max_size: 最大文件大小 (bytes)
        allowed_types: 允许的 MIME 类型列表
        multiple: 是否支持多文件上传
        max_count: 多文件上传时的最大文件数量

    Example:
        # 单文件上传（新的统一语法）
        @post_api(url="/upload")
        async def upload(
            self,
            avatar: File = File(max_size=5*1024*1024),  # 5MB
        ):
            print(avatar.filename)
            print(avatar.size)
            avatar.save('/uploads/')

        # 使用 as_required() 快捷方法
        @post_api(url="/upload-required")
        async def upload_required(
            self,
            avatar: File = File.as_required(max_size=5*1024*1024),
        ):
            pass

        # 带类型校验
        @post_api(url="/upload-image")
        async def upload_image(
            self,
            image: File = File(allowed_types=['image/png', 'image/jpeg', 'image/*']),
        ):
            pass

        # 多文件上传
        @post_api(url="/upload-multiple")
        async def upload_multiple(
            self,
            files: File = File(multiple=True, max_count=10),
        ):
            for f in files:
                print(f.filename)
    """
    _source = 'file'

    # 扩展 __slots__
    __slots__ = ('max_size', 'allowed_types', 'multiple', 'max_count', 'min_size')

    def __init__(
        self,
        *,
        name: str = None,
        required: bool = True,
        description: str = '',
        alias: str = None,
        max_size: Optional[int] = None,
        min_size: Optional[int] = None,
        allowed_types: Optional[List[str]] = None,
        multiple: bool = False,
        max_count: Optional[int] = None,
    ):
        """初始化文件参数

        Args:
            name: 参数名称
            required: 是否必填
            description: 参数描述
            alias: 别名 (表单字段名)
            max_size: 最大文件大小 (bytes)
            min_size: 最小文件大小 (bytes)
            allowed_types: 允许的 MIME 类型列表，支持通配符如 'image/*'
            multiple: 是否支持多文件上传
            max_count: 多文件上传时的最大文件数量
        """
        super().__init__(
            type_=bytes,  # 文件内容为 bytes
            name=name,
            required=required,
            default=UNSET,
            description=description,
            alias=alias,
        )
        self.max_size = max_size
        self.min_size = min_size
        self.allowed_types = allowed_types or []
        self.multiple = multiple
        self.max_count = max_count

    def validate_file(self, file_info) -> None:
        """校验文件

        Args:
            file_info: FileInfo 实例

        Raises:
            ValueError: 校验失败
        """
        # 校验文件大小
        if self.max_size is not None and file_info.size > self.max_size:
            raise ValueError(
                f"File '{file_info.filename}' size ({file_info.size} bytes) "
                f"exceeds maximum allowed size ({self.max_size} bytes)"
            )

        if self.min_size is not None and file_info.size < self.min_size:
            raise ValueError(
                f"File '{file_info.filename}' size ({file_info.size} bytes) "
                f"is below minimum required size ({self.min_size} bytes)"
            )

        # 校验 MIME 类型
        if self.allowed_types:
            if not self._match_content_type(file_info.content_type):
                raise ValueError(
                    f"File '{file_info.filename}' type '{file_info.content_type}' "
                    f"is not allowed. Allowed types: {self.allowed_types}"
                )

    def validate_file_list(self, file_list) -> None:
        """校验文件列表

        Args:
            file_list: FileList 实例

        Raises:
            ValueError: 校验失败
        """
        # 校验文件数量
        if self.max_count is not None and len(file_list) > self.max_count:
            raise ValueError(
                f"Number of files ({len(file_list)}) exceeds maximum allowed ({self.max_count})"
            )

        # 校验每个文件
        for file_info in file_list:
            self.validate_file(file_info)

    def _match_content_type(self, content_type: str) -> bool:
        """检查 MIME 类型是否匹配允许列表

        Args:
            content_type: 文件的 MIME 类型

        Returns:
            是否匹配
        """
        for pattern in self.allowed_types:
            if pattern == '*/*' or pattern == '*':
                return True
            if pattern.endswith('/*'):
                # 通配符匹配
                prefix = pattern[:-1]  # 'image/'
                if content_type.startswith(prefix):
                    return True
            elif content_type == pattern:
                return True
        return False

    def __repr__(self) -> str:
        parts = ["File("]
        if self.name:
            parts.append(f"name={self.name!r}, ")
        if not self.required:
            parts.append("required=False, ")
        if self.max_size:
            parts.append(f"max_size={self.max_size}, ")
        if self.min_size:
            parts.append(f"min_size={self.min_size}, ")
        if self.allowed_types:
            parts.append(f"allowed_types={self.allowed_types!r}, ")
        if self.multiple:
            parts.append("multiple=True, ")
        if self.max_count:
            parts.append(f"max_count={self.max_count}, ")
        result = "".join(parts)
        if result.endswith(", "):
            result = result[:-2]
        return result + ")"

    @classmethod
    def as_required(
        cls,
        *,
        name: str = None,
        description: str = '',
        alias: str = None,
        max_size: Optional[int] = None,
        min_size: Optional[int] = None,
        allowed_types: Optional[List[str]] = None,
        multiple: bool = False,
        max_count: Optional[int] = None,
    ) -> 'File':
        """创建必填文件参数的快捷方法

        Example:
            # 以下两种写法等价
            avatar: File = File.as_required(max_size=5*1024*1024)
            avatar: File = File(required=True, max_size=5*1024*1024)

        Returns:
            设置了 required=True 的 File 实例
        """
        return cls(
            name=name,
            required=True,
            description=description,
            alias=alias,
            max_size=max_size,
            min_size=min_size,
            allowed_types=allowed_types,
            multiple=multiple,
            max_count=max_count,
        )




# -*- coding: utf-8 -*-
"""Cullinan JSON Codec

JSON 请求体解码器和响应编码器实现。

Author: Plumeink
"""

import json
from typing import Any, Dict

from .base import BodyCodec, ResponseCodec
from .errors import DecodeError, EncodeError


class JsonBodyCodec(BodyCodec):
    """JSON 请求体解码器

    支持的 Content-Type:
    - application/json
    - text/json
    - application/json; charset=utf-8
    """

    content_types = ['application/json', 'text/json']
    priority = 10  # 高优先级

    def decode(self, body: bytes, charset: str = 'utf-8') -> Dict[str, Any]:
        """解码 JSON 请求体

        Args:
            body: 原始请求体字节
            charset: 字符编码

        Returns:
            解码后的字典 (空请求体返回空字典)

        Raises:
            DecodeError: JSON 解析失败
        """
        if not body:
            return {}

        try:
            decoded = body.decode(charset)
            result = json.loads(decoded)

            # 确保返回字典
            if isinstance(result, dict):
                return result
            # 非字典值包装 (如 JSON 数组或标量)
            return {'_value': result}

        except UnicodeDecodeError as e:
            raise DecodeError(
                f"Failed to decode body with charset '{charset}': {e}",
                content_type='application/json',
                body_preview=body
            )
        except json.JSONDecodeError as e:
            raise DecodeError(
                f"Invalid JSON: {e}",
                content_type='application/json',
                body_preview=body
            )

    def encode(self, data: Dict[str, Any], charset: str = 'utf-8') -> bytes:
        """编码为 JSON

        Args:
            data: 要编码的字典
            charset: 字符编码

        Returns:
            JSON 字节串
        """
        try:
            return json.dumps(data, ensure_ascii=False).encode(charset)
        except (TypeError, ValueError) as e:
            raise EncodeError(
                f"Failed to encode as JSON: {e}",
                data_type=type(data)
            )


class JsonResponseCodec(ResponseCodec):
    """JSON 响应编码器

    将响应数据编码为 JSON 格式。
    """

    content_type = 'application/json'
    accept_types = ['application/json', 'text/json', '*/*']
    priority = 10  # 高优先级

    def encode(self, data: Any, charset: str = 'utf-8') -> bytes:
        """编码响应数据为 JSON

        Args:
            data: 响应数据 (可以是任意 JSON 可序列化类型)
            charset: 字符编码

        Returns:
            JSON 字节串

        Raises:
            EncodeError: 序列化失败
        """
        try:
            # 使用 default=str 处理不可序列化的类型
            return json.dumps(
                data,
                ensure_ascii=False,
                default=str
            ).encode(charset)
        except Exception as e:
            raise EncodeError(
                f"Failed to encode response as JSON: {e}",
                data_type=type(data)
            )




# -*- coding: utf-8 -*-
"""测试 Header 和 RawBody 语法"""

import sys
import os
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from cullinan.params import Header, ParamResolver, RawBody

# 测试1: RawBody 不带括号（推荐，与 DynamicBody 一致）
def handle_webhook_v1(
    self,
    sign: Header(str, alias="X-Hub-Signature-256"),
    event: Header(str, alias="X-GitHub-Event"),
    raw_body: RawBody,  # 不带括号
):
    pass

# 测试2: RawBody 带括号（也支持）
def handle_webhook_v2(
    self,
    sign: Header(str, alias="X-Hub-Signature-256"),
    event: Header(str, alias="X-GitHub-Event"),
    raw_body: RawBody(),  # 带括号
):
    pass

print("=" * 60)
print("测试 Header 和 RawBody 语法")
print("=" * 60)

# Test 1: RawBody 不带括号
print("\n1. 测试 RawBody 不带括号（推荐）:")
config = ParamResolver.analyze_params(handle_webhook_v1)
assert config['raw_body']['source'] == 'raw_body', f"source 错误: {config['raw_body']['source']}"
assert config['raw_body']['type'] == bytes, f"type 错误: {config['raw_body']['type']}"
print("   raw_body: source=raw_body, type=bytes ✓")

# Test 2: RawBody 带括号
print("\n2. 测试 RawBody 带括号（也支持）:")
config = ParamResolver.analyze_params(handle_webhook_v2)
assert config['raw_body']['source'] == 'raw_body', f"source 错误: {config['raw_body']['source']}"
assert config['raw_body']['type'] == bytes, f"type 错误: {config['raw_body']['type']}"
print("   raw_body: source=raw_body, type=bytes ✓")

# Test 3: 实际解析
print("\n3. 测试实际解析:")

class MockRequest:
    body = b'{"action": "push", "ref": "refs/heads/main"}'

headers_dict = {
    'X-Hub-Signature-256': 'sha256=abc123',
    'X-Github-Event': 'push',
}

resolved = ParamResolver.resolve(
    func=handle_webhook_v1,
    request=MockRequest(),
    url_params={},
    query_params={},
    body_data={'action': 'push'},
    headers=headers_dict,
    files={},
)

print(f"   sign = {resolved.get('sign')}")
print(f"   event = {resolved.get('event')}")
print(f"   raw_body = {resolved.get('raw_body')[:30]}...")
print(f"   raw_body type = {type(resolved.get('raw_body'))}")

assert resolved.get('sign') == 'sha256=abc123'
assert resolved.get('event') == 'push'
assert resolved.get('raw_body') == b'{"action": "push", "ref": "refs/heads/main"}'
assert isinstance(resolved.get('raw_body'), bytes)

print("\n" + "=" * 60)
print("所有测试通过!")
print("=" * 60)




# -*- coding: utf-8 -*-
"""Cullinan Model Handlers

可插拔的模型处理器模块。

提供统一的模型解析接口，支持 dataclass、Pydantic 等多种模型库。

Author: Plumeink
"""

from typing import Any, Dict, List, Optional, Type

from .base import ModelHandler, ModelHandlerError
from .dataclass_handler import DataclassHandler


class ModelHandlerRegistry:
    """模型处理器注册表

    管理所有已注册的模型处理器，根据类型自动选择合适的处理器。

    Features:
    - 自动发现可用处理器
    - 按优先级排序
    - 支持手动注册/注销
    - 线程安全的单例模式

    Example:
        # 获取注册表
        registry = get_model_handler_registry()

        # 获取能处理某类型的处理器
        handler = registry.get_handler(MyDataclass)
        if handler:
            instance = handler.resolve(MyDataclass, data)

        # 注册自定义处理器
        registry.register(MyCustomHandler())
    """

    def __init__(self):
        self._handlers: List[ModelHandler] = []
        self._initialized = False

    def register(self, handler: ModelHandler) -> None:
        """注册处理器

        Args:
            handler: 模型处理器实例
        """
        if handler not in self._handlers:
            self._handlers.append(handler)
            # 按优先级降序排序
            self._handlers.sort(key=lambda h: h.priority, reverse=True)

    def unregister(self, handler: ModelHandler) -> bool:
        """注销处理器

        Args:
            handler: 模型处理器实例

        Returns:
            True 如果成功注销
        """
        if handler in self._handlers:
            self._handlers.remove(handler)
            return True
        return False

    def unregister_by_name(self, name: str) -> bool:
        """按名称注销处理器

        Args:
            name: 处理器名称

        Returns:
            True 如果成功注销
        """
        for handler in self._handlers[:]:
            if handler.name == name:
                self._handlers.remove(handler)
                return True
        return False

    def get_handler(self, type_: Type) -> Optional[ModelHandler]:
        """获取能处理指定类型的处理器

        按优先级顺序查找，返回第一个能处理的处理器。

        Args:
            type_: 类型

        Returns:
            处理器实例，如果没有找到返回 None
        """
        self._ensure_initialized()

        for handler in self._handlers:
            try:
                if handler.can_handle(type_):
                    return handler
            except Exception:
                continue
        return None

    def can_handle(self, type_: Type) -> bool:
        """检查是否有处理器能处理指定类型

        Args:
            type_: 类型

        Returns:
            True 如果有处理器能处理
        """
        return self.get_handler(type_) is not None

    def resolve(self, model_class: Type, data: Dict[str, Any]) -> Any:
        """使用合适的处理器解析数据

        Args:
            model_class: 模型类
            data: 数据字典

        Returns:
            模型实例

        Raises:
            ModelHandlerError: 没有处理器或解析失败
        """
        handler = self.get_handler(model_class)
        if handler is None:
            raise ModelHandlerError(
                f"No handler found for {model_class}",
                model_class=model_class,
            )
        return handler.resolve(model_class, data)

    @property
    def handlers(self) -> List[ModelHandler]:
        """获取所有已注册的处理器"""
        self._ensure_initialized()
        return self._handlers.copy()

    def get_handler_names(self) -> List[str]:
        """获取所有处理器名称"""
        self._ensure_initialized()
        return [h.name for h in self._handlers]

    def _ensure_initialized(self) -> None:
        """确保已初始化"""
        if not self._initialized:
            self._auto_discover()
            self._initialized = True

    def _auto_discover(self) -> None:
        """自动发现并注册可用的处理器"""
        # 1. 注册内置的 dataclass 处理器
        self.register(DataclassHandler())

        # 2. 尝试加载 Pydantic 处理器
        try:
            from .pydantic_handler import PydanticHandler
            self.register(PydanticHandler())
        except ImportError:
            pass  # Pydantic 未安装，跳过

    def reset(self) -> None:
        """重置注册表（用于测试）"""
        self._handlers.clear()
        self._initialized = False


# 全局注册表实例
_registry: Optional[ModelHandlerRegistry] = None


def get_model_handler_registry() -> ModelHandlerRegistry:
    """获取全局模型处理器注册表

    Returns:
        ModelHandlerRegistry 实例
    """
    global _registry
    if _registry is None:
        _registry = ModelHandlerRegistry()
    return _registry


def reset_model_handler_registry() -> None:
    """重置全局注册表（用于测试）"""
    global _registry
    if _registry is not None:
        _registry.reset()
    _registry = None


# 导出
__all__ = [
    'ModelHandler',
    'ModelHandlerError',
    'ModelHandlerRegistry',
    'DataclassHandler',
    'get_model_handler_registry',
    'reset_model_handler_registry',
]

# 尝试导出 PydanticHandler（如果可用）
try:
    from .pydantic_handler import PydanticHandler
    __all__.append('PydanticHandler')
except ImportError:
    pass




# -*- coding: utf-8 -*-
"""Cullinan Codec Module

提供请求体和响应体的编解码功能。

支持的格式:
- JSON (application/json)
- Form (application/x-www-form-urlencoded)
- 可扩展自定义格式

Author: Plumeink
"""

from .base import BodyCodec, ResponseCodec
from .errors import CodecError, DecodeError, EncodeError
from .json_codec import JsonBodyCodec, JsonResponseCodec
from .form_codec import FormBodyCodec
from .registry import (
    CodecRegistry,
    get_codec_registry,
    reset_codec_registry,
)

__all__ = [
    # 抽象基类
    'BodyCodec',
    'ResponseCodec',

    # 错误类型
    'CodecError',
    'DecodeError',
    'EncodeError',

    # 内置 Codec
    'JsonBodyCodec',
    'JsonResponseCodec',
    'FormBodyCodec',

    # 注册表
    'CodecRegistry',
    'get_codec_registry',
    'reset_codec_registry',
]




# -*- coding: utf-8 -*-
"""Cullinan Codec Module Tests

测试 Codec 模块的编解码功能。

Author: Plumeink
"""

import unittest
import json

from cullinan.codec import (
    BodyCodec,
    ResponseCodec,
    JsonBodyCodec,
    JsonResponseCodec,
    FormBodyCodec,
    DecodeError,
    EncodeError,
    CodecRegistry,
    get_codec_registry,
    reset_codec_registry,
)


class TestJsonBodyCodec(unittest.TestCase):
    """测试 JSON 请求体解码"""

    def setUp(self):
        self.codec = JsonBodyCodec()

    def test_decode_empty_body(self):
        """空请求体应返回空字典"""
        result = self.codec.decode(b'')
        self.assertEqual(result, {})

    def test_decode_valid_json(self):
        """正常 JSON 解码"""
        body = b'{"name": "test", "age": 18}'
        result = self.codec.decode(body)
        self.assertEqual(result, {"name": "test", "age": 18})

    def test_decode_json_array(self):
        """JSON 数组应被包装"""
        body = b'[1, 2, 3]'
        result = self.codec.decode(body)
        self.assertEqual(result, {"_value": [1, 2, 3]})

    def test_decode_invalid_json(self):
        """无效 JSON 应抛出 DecodeError"""
        body = b'invalid json'
        with self.assertRaises(DecodeError):
            self.codec.decode(body)

    def test_decode_with_charset(self):
        """带字符编码的解码"""
        body = '{"name": "中文"}'.encode('utf-8')
        result = self.codec.decode(body, charset='utf-8')
        self.assertEqual(result, {"name": "中文"})

    def test_supports_content_type(self):
        """Content-Type 支持检测"""
        self.assertTrue(JsonBodyCodec.supports('application/json'))
        self.assertTrue(JsonBodyCodec.supports('application/json; charset=utf-8'))
        self.assertTrue(JsonBodyCodec.supports('text/json'))
        self.assertFalse(JsonBodyCodec.supports('text/plain'))


class TestJsonResponseCodec(unittest.TestCase):
    """测试 JSON 响应编码"""

    def setUp(self):
        self.codec = JsonResponseCodec()

    def test_encode_dict(self):
        """编码字典"""
        data = {"name": "test", "age": 18}
        result = self.codec.encode(data)
        self.assertEqual(json.loads(result.decode('utf-8')), data)

    def test_encode_with_chinese(self):
        """编码中文"""
        data = {"name": "中文"}
        result = self.codec.encode(data)
        self.assertIn("中文", result.decode('utf-8'))

    def test_get_content_type(self):
        """获取 Content-Type"""
        ct = self.codec.get_content_type()
        self.assertIn('application/json', ct)
        self.assertIn('utf-8', ct)


class TestFormBodyCodec(unittest.TestCase):
    """测试 Form 请求体解码"""

    def setUp(self):
        self.codec = FormBodyCodec()

    def test_decode_empty_body(self):
        """空请求体应返回空字典"""
        result = self.codec.decode(b'')
        self.assertEqual(result, {})

    def test_decode_simple_form(self):
        """简单表单解码"""
        body = b'name=test&age=18'
        result = self.codec.decode(body)
        self.assertEqual(result, {"name": "test", "age": "18"})

    def test_decode_multi_value(self):
        """多值字段"""
        body = b'tags=a&tags=b&tags=c'
        result = self.codec.decode(body)
        self.assertEqual(result, {"tags": ["a", "b", "c"]})

    def test_supports_content_type(self):
        """Content-Type 支持检测"""
        self.assertTrue(FormBodyCodec.supports('application/x-www-form-urlencoded'))
        self.assertFalse(FormBodyCodec.supports('application/json'))


class TestCodecRegistry(unittest.TestCase):
    """测试 Codec 注册表"""

    def setUp(self):
        reset_codec_registry()
        self.registry = get_codec_registry()

    def tearDown(self):
        reset_codec_registry()

    def test_default_codecs_registered(self):
        """默认 Codec 应已注册"""
        body_codecs = self.registry.list_body_codecs()
        self.assertTrue(any(c == JsonBodyCodec for c in body_codecs))
        self.assertTrue(any(c == FormBodyCodec for c in body_codecs))

    def test_get_body_codec_json(self):
        """获取 JSON Codec"""
        codec = self.registry.get_body_codec('application/json')
        self.assertIsInstance(codec, JsonBodyCodec)

    def test_get_body_codec_form(self):
        """获取 Form Codec"""
        codec = self.registry.get_body_codec('application/x-www-form-urlencoded')
        self.assertIsInstance(codec, FormBodyCodec)

    def test_decode_body_json(self):
        """通过注册表解码 JSON"""
        body = b'{"test": true}'
        result = self.registry.decode_body(body, 'application/json')
        self.assertEqual(result, {"test": True})

    def test_decode_body_form(self):
        """通过注册表解码 Form"""
        body = b'key=value'
        result = self.registry.decode_body(body, 'application/x-www-form-urlencoded')
        self.assertEqual(result, {"key": "value"})

    def test_encode_response(self):
        """通过注册表编码响应"""
        data = {"status": "ok"}
        encoded, content_type = self.registry.encode_response(data)
        self.assertIn('application/json', content_type)
        self.assertEqual(json.loads(encoded.decode('utf-8')), data)


if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-
"""可插拔模型处理器测试"""

import sys
import os
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from dataclasses import dataclass
from typing import Optional


def test_registry_basic():
    """测试注册表基本功能"""
    print("1. 测试注册表基本功能...")

    from cullinan.params.model_handlers import (
        get_model_handler_registry,
        reset_model_handler_registry,
        DataclassHandler,
    )

    # 重置注册表
    reset_model_handler_registry()

    # 获取注册表
    registry = get_model_handler_registry()

    # 检查自动发现
    handlers = registry.get_handler_names()
    assert 'dataclass' in handlers, f"Expected 'dataclass' in {handlers}"

    print(f"   已注册的处理器: {handlers}")
    print("   基本功能测试通过")
    return True


def test_dataclass_handler():
    """测试 dataclass 处理器"""
    print("2. 测试 dataclass 处理器...")

    from cullinan.params.model_handlers import (
        get_model_handler_registry,
        reset_model_handler_registry,
    )

    reset_model_handler_registry()
    registry = get_model_handler_registry()

    @dataclass
    class User:
        name: str
        age: int = 0
        email: Optional[str] = None

    # 检查可处理
    handler = registry.get_handler(User)
    assert handler is not None
    assert handler.name == 'dataclass'

    # 测试解析
    data = {'name': 'John', 'age': '25', 'email': 'john@example.com'}
    user = handler.resolve(User, data)

    assert user.name == 'John'
    assert user.age == 25
    assert user.email == 'john@example.com'

    # 测试 to_dict
    result = handler.to_dict(user)
    assert result['name'] == 'John'
    assert result['age'] == 25

    print("   dataclass 处理器测试通过")
    return True


def test_param_resolver_with_dataclass():
    """测试 ParamResolver 使用注册表"""
    print("3. 测试 ParamResolver 使用模型处理器...")

    from cullinan.params import ParamResolver
    from cullinan.params.model_handlers import reset_model_handler_registry

    reset_model_handler_registry()

    @dataclass
    class CreateUserRequest:
        name: str
        age: int = 0

    def handler(self, user: CreateUserRequest):
        pass

    # 分析参数
    config = ParamResolver.analyze_params(handler)

    assert 'user' in config
    assert config['user']['source'] == 'body'
    assert config['user'].get('model_handler') is not None
    assert config['user']['model_handler'].name == 'dataclass'

    print("   ParamResolver 模型处理器集成测试通过")
    return True


def test_custom_handler():
    """测试自定义处理器注册"""
    print("4. 测试自定义处理器注册...")

    from cullinan.params.model_handlers import (
        ModelHandler,
        ModelHandlerError,
        get_model_handler_registry,
        reset_model_handler_registry,
    )

    reset_model_handler_registry()

    # 创建自定义处理器
    class CustomModel:
        """自定义模型类"""
        def __init__(self, data):
            self.data = data

    class CustomHandler(ModelHandler):
        priority = 100  # 高优先级
        name = "custom"

        def can_handle(self, type_):
            return type_ is CustomModel

        def resolve(self, model_class, data):
            return CustomModel(data)

        def to_dict(self, instance):
            return instance.data

    # 注册自定义处理器
    registry = get_model_handler_registry()
    registry.register(CustomHandler())

    # 检查注册
    assert 'custom' in registry.get_handler_names()

    # 检查优先级（custom 应该在最前面）
    handlers = registry.handlers
    assert handlers[0].name == 'custom'

    # 测试解析
    handler = registry.get_handler(CustomModel)
    assert handler.name == 'custom'

    instance = handler.resolve(CustomModel, {'key': 'value'})
    assert instance.data == {'key': 'value'}

    print("   自定义处理器测试通过")
    return True


def test_pydantic_handler_optional():
    """测试 Pydantic 处理器（可选）"""
    print("5. 测试 Pydantic 处理器...")

    from cullinan.params.model_handlers import (
        get_model_handler_registry,
        reset_model_handler_registry,
    )

    reset_model_handler_registry()
    registry = get_model_handler_registry()

    handlers = registry.get_handler_names()

    if 'pydantic' in handlers:
        print("   Pydantic 已安装，测试 Pydantic 处理器...")

        from pydantic import BaseModel

        class PydanticUser(BaseModel):
            name: str
            age: int = 0

        handler = registry.get_handler(PydanticUser)
        assert handler is not None
        assert handler.name == 'pydantic'

        # 测试解析
        data = {'name': 'Jane', 'age': 30}
        user = handler.resolve(PydanticUser, data)

        assert user.name == 'Jane'
        assert user.age == 30

        print("   Pydantic 处理器测试通过")
    else:
        print("   Pydantic 未安装，跳过 Pydantic 测试")

    return True


def test_handler_priority():
    """测试处理器优先级"""
    print("6. 测试处理器优先级...")

    from cullinan.params.model_handlers import (
        get_model_handler_registry,
        reset_model_handler_registry,
    )

    reset_model_handler_registry()
    registry = get_model_handler_registry()

    handlers = registry.handlers

    # 检查优先级排序（降序）
    for i in range(len(handlers) - 1):
        assert handlers[i].priority >= handlers[i+1].priority, \
            f"Handler priority not sorted: {handlers[i].name}({handlers[i].priority}) < {handlers[i+1].name}({handlers[i+1].priority})"

    print(f"   处理器优先级顺序: {[(h.name, h.priority) for h in handlers]}")
    print("   处理器优先级测试通过")
    return True


def run_all_tests():
    """运行所有测试"""
    print("=" * 60)
    print("可插拔模型处理器架构测试")
    print("=" * 60)
    print()

    tests = [
        test_registry_basic,
        test_dataclass_handler,
        test_param_resolver_with_dataclass,
        test_custom_handler,
        test_pydantic_handler_optional,
        test_handler_priority,
    ]

    passed = 0
    failed = 0

    for test in tests:
        try:
            test()
            passed += 1
        except AssertionError as e:
            print(f"   FAILED: {e}")
            failed += 1
        except Exception as e:
            print(f"   ERROR: {type(e).__name__}: {e}")
            import traceback
            traceback.print_exc()
            failed += 1

    print()
    print("=" * 60)
    print(f"测试结果: {passed} 通过, {failed} 失败")
    print("=" * 60)

    return failed == 0


if __name__ == '__main__':
    success = run_all_tests()
    sys.exit(0 if success else 1)




# -*- coding: utf-8 -*-
"""Cullinan Dataclass Validators

为 dataclass 提供字段级校验装饰器。

Author: Plumeink
"""

from typing import Any, Callable, Dict, List, Type, Union
from functools import wraps
import dataclasses


# 存储每个 dataclass 的字段校验器
_field_validators: Dict[Type, Dict[str, List[Callable]]] = {}


class FieldValidationError(Exception):
    """字段校验错误

    Attributes:
        field: 字段名
        value: 字段值
        message: 错误消息
    """

    def __init__(self, field: str, value: Any, message: str):
        self.field = field
        self.value = value
        self.message = message
        super().__init__(f"Field '{field}' validation failed: {message}")

    def to_dict(self) -> dict:
        return {
            'field': self.field,
            'value': self.value,
            'message': self.message,
        }


def field_validator(*fields: str, mode: str = 'after'):
    """字段校验器装饰器

    用于在 dataclass 中定义字段级别的校验逻辑。

    Args:
        *fields: 要校验的字段名列表
        mode: 校验模式
            - 'after': 在类型转换后校验（默认）
            - 'before': 在类型转换前校验

    Example:
        from dataclasses import dataclass
        from cullinan.params import field_validator

        @dataclass
        class CreateUserRequest:
            name: str
            email: str
            age: int = 0

            @field_validator('email')
            @classmethod
            def validate_email(cls, v):
                if '@' not in v:
                    raise ValueError('Invalid email format')
                return v

            @field_validator('age')
            @classmethod
            def validate_age(cls, v):
                if v < 0 or v > 150:
                    raise ValueError('Age must be between 0 and 150')
                return v

            @field_validator('name', 'email')
            @classmethod
            def strip_strings(cls, v):
                if isinstance(v, str):
                    return v.strip()
                return v
    """
    def decorator(func: Callable) -> Callable:
        # 检查是否是 classmethod
        if isinstance(func, classmethod):
            # 获取实际函数
            actual_func = func.__func__
            # 标记校验器属性
            actual_func._is_field_validator = True
            actual_func._validator_fields = fields
            actual_func._validator_mode = mode
            # 返回原始 classmethod，保持其类型
            return func
        else:
            # 普通函数
            func._is_field_validator = True
            func._validator_fields = fields
            func._validator_mode = mode
            return func

    return decorator


def register_validators(cls: Type) -> None:
    """注册 dataclass 的所有字段校验器

    Args:
        cls: dataclass 类
    """
    if not dataclasses.is_dataclass(cls):
        return

    validators = {}

    # 扫描类中的校验器方法
    for name in dir(cls):
        if name.startswith('_'):
            continue

        try:
            # 使用 __dict__ 直接获取，避免描述符协议
            if name in cls.__dict__:
                method = cls.__dict__[name]
            else:
                method = getattr(cls, name, None)
        except Exception:
            continue

        if method is None:
            continue

        # 处理 classmethod 包装
        actual_method = method
        if isinstance(method, classmethod):
            actual_method = method.__func__

        # 检查是否是校验器
        if hasattr(actual_method, '_is_field_validator') and actual_method._is_field_validator:
            for field in actual_method._validator_fields:
                if field not in validators:
                    validators[field] = []
                validators[field].append({
                    'func': actual_method,
                    'mode': actual_method._validator_mode,
                })

    if validators:
        _field_validators[cls] = validators


def get_validators(cls: Type) -> Dict[str, List[dict]]:
    """获取 dataclass 的字段校验器

    Args:
        cls: dataclass 类

    Returns:
        字段名到校验器列表的映射
    """
    if cls not in _field_validators:
        register_validators(cls)
    return _field_validators.get(cls, {})


def validate_field(cls: Type, field: str, value: Any, mode: str = 'after') -> Any:
    """对字段值执行校验

    Args:
        cls: dataclass 类
        field: 字段名
        value: 字段值
        mode: 校验模式

    Returns:
        校验/转换后的值

    Raises:
        FieldValidationError: 校验失败
    """
    validators = get_validators(cls)
    field_validators = validators.get(field, [])

    for validator in field_validators:
        if validator['mode'] != mode:
            continue

        try:
            value = validator['func'](cls, value)
        except ValueError as e:
            raise FieldValidationError(field, value, str(e))
        except Exception as e:
            raise FieldValidationError(field, value, str(e))

    return value


def validate_dataclass(instance) -> None:
    """校验整个 dataclass 实例

    Args:
        instance: dataclass 实例

    Raises:
        FieldValidationError: 校验失败
    """
    cls = type(instance)

    if not dataclasses.is_dataclass(cls):
        return

    validators = get_validators(cls)

    for field in dataclasses.fields(instance):
        field_name = field.name
        value = getattr(instance, field_name)

        # 执行 after 模式的校验器
        if field_name in validators:
            for validator in validators[field_name]:
                if validator['mode'] == 'after':
                    try:
                        func = validator['func']
                        # 处理 classmethod - 需要使用类作为第一个参数
                        if isinstance(func, classmethod):
                            new_value = func.__func__(cls, value)
                        else:
                            # 普通函数/方法
                            new_value = func(cls, value)
                        setattr(instance, field_name, new_value)
                    except ValueError as e:
                        raise FieldValidationError(field_name, value, str(e))


class validated_dataclass:
    """带自动校验的 dataclass 装饰器

    包装 @dataclass，自动在实例化后执行字段校验。

    Example:
        from cullinan.params import validated_dataclass, field_validator

        @validated_dataclass
        class CreateUserRequest:
            name: str
            email: str
            age: int = 0

            @field_validator('email')
            @classmethod
            def validate_email(cls, v):
                if '@' not in v:
                    raise ValueError('Invalid email format')
                return v
    """

    def __new__(cls, wrapped_cls=None, **kwargs):
        """支持 @validated_dataclass 和 @validated_dataclass() 两种用法"""
        if wrapped_cls is not None:
            # @validated_dataclass 不带括号
            return cls._wrap_class(wrapped_cls, kwargs)
        else:
            # @validated_dataclass() 带括号
            def decorator(c):
                return cls._wrap_class(c, kwargs)
            return decorator

    @staticmethod
    def _wrap_class(cls, kwargs=None):
        """包装类"""
        if kwargs is None:
            kwargs = {}

        # 应用 @dataclass
        if not dataclasses.is_dataclass(cls):
            cls = dataclasses.dataclass(cls, **kwargs)

        # 注册校验器（在 @dataclass 之后）
        register_validators(cls)

        # 保存原始的 __init__
        original_init = cls.__init__

        def new_init(self, *args, **kw):
            # 调用原始的 __init__
            original_init(self, *args, **kw)
            # 执行校验
            validate_dataclass(self)

        cls.__init__ = new_init

        return cls


def clear_validators():
    """清除所有已注册的校验器（用于测试）"""
    _field_validators.clear()




# -*- coding: utf-8 -*-
"""Cullinan 全面测试套件

包括：
1. 核心功能测试
2. IoC/DI 测试
3. 参数系统测试
4. Codec 测试
5. 性能基准测试
6. 回归测试

Author: Plumeink
"""

import sys
import os
import time
import traceback

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# 测试结果收集
test_results = {
    'passed': 0,
    'failed': 0,
    'errors': []
}

def test(name):
    """测试装饰器"""
    def decorator(func):
        def wrapper():
            try:
                print(f"  Testing: {name}...", end=" ")
                start = time.perf_counter()
                func()
                elapsed = (time.perf_counter() - start) * 1000
                print(f"PASSED ({elapsed:.2f}ms)")
                test_results['passed'] += 1
                return True
            except AssertionError as e:
                print(f"FAILED: {e}")
                test_results['failed'] += 1
                test_results['errors'].append((name, str(e)))
                return False
            except Exception as e:
                print(f"ERROR: {e}")
                test_results['failed'] += 1
                test_results['errors'].append((name, traceback.format_exc()))
                return False
        return wrapper
    return decorator


# ============================================================================
# 1. IoC/DI 核心测试
# ============================================================================
print("\n" + "=" * 70)
print("1. IoC/DI 核心测试")
print("=" * 70)

@test("ApplicationContext 基本功能")
def test_app_context_basic():
    from cullinan.core import ApplicationContext
    from cullinan.core.pending import PendingRegistry
    from cullinan.core import service

    PendingRegistry.reset()

    @service
    class TestService:
        def get_name(self):
            return 'test'

    ctx = ApplicationContext()
    ctx.refresh()

    svc = ctx.get('TestService')
    assert svc is not None, "Service should not be None"
    assert svc.get_name() == 'test', "Service name should be 'test'"

test_app_context_basic()

@test("Inject 描述符")
def test_inject_descriptor():
    from cullinan.core import Inject, InjectByName
    from cullinan.core import service, ApplicationContext
    from cullinan.core.pending import PendingRegistry

    # 清理状态
    PendingRegistry.reset()

    @service
    class DepService:
        def get_value(self):
            return 42

    @service
    class MainService:
        dep: DepService = Inject()

        def compute(self):
            return self.dep.get_value() * 2

    ctx = ApplicationContext()
    ctx.refresh()

    main = ctx.get('MainService')
    assert main is not None, "MainService should not be None"
    assert main.compute() == 84, "Compute should return 84"

test_inject_descriptor()

@test("InjectByName 描述符")
def test_inject_by_name():
    from cullinan.core import InjectByName
    from cullinan.core import service, ApplicationContext
    from cullinan.core.pending import PendingRegistry

    PendingRegistry.reset()

    @service
    class NamedService:
        def get_name(self):
            return "NamedService"

    @service
    class ConsumerService:
        named = InjectByName('NamedService')

        def use(self):
            return self.named.get_name()

    ctx = ApplicationContext()
    ctx.refresh()

    consumer = ctx.get('ConsumerService')
    assert consumer.use() == "NamedService"

test_inject_by_name()

@test("条件注册 ConditionalOnClass")
def test_conditional_on_class():
    from cullinan.core import service, ApplicationContext
    from cullinan.core.conditions import ConditionalOnClass
    from cullinan.core.pending import PendingRegistry

    PendingRegistry.reset()

    @service
    @ConditionalOnClass('json')  # json 模块存在
    class JsonService2:
        pass

    @service
    @ConditionalOnClass('nonexistent_module_xyz_abc')  # 不存在的模块
    class NonExistentService2:
        pass

    ctx = ApplicationContext()
    ctx.refresh()

    assert ctx.get('JsonService2') is not None, "JsonService2 should exist"
    # 不存在的模块条件下的服务可能抛异常或返回 None，这里只检查 JsonService2 存在

test_conditional_on_class()


# ============================================================================
# 2. 参数系统测试
# ============================================================================
print("\n" + "=" * 70)
print("2. 参数系统测试")
print("=" * 70)

@test("Path 参数解析")
def test_path_param():
    from cullinan.params import Path, ParamResolver

    def handler(self, id: Path(int), name: Path(str)):
        pass

    result = ParamResolver.resolve(
        func=handler,
        request=None,
        url_params={'id': '123', 'name': 'test'}
    )

    assert result['id'] == 123, f"Expected 123, got {result['id']}"
    assert type(result['id']) is int, f"Expected int, got {type(result['id'])}"
    assert result['name'] == 'test'

test_path_param()

@test("Query 参数解析和默认值")
def test_query_param():
    from cullinan.params import Query, ParamResolver

    def handler(self, page: Query(int, default=1), size: Query(int, default=10)):
        pass

    # 有值
    result = ParamResolver.resolve(
        func=handler,
        request=None,
        query_params={'page': '5', 'size': '20'}
    )
    assert result['page'] == 5
    assert result['size'] == 20

    # 默认值
    result = ParamResolver.resolve(
        func=handler,
        request=None,
        query_params={}
    )
    assert result['page'] == 1
    assert result['size'] == 10

test_query_param()

@test("Body 参数解析")
def test_body_param():
    from cullinan.params import Body, ParamResolver

    def handler(self, name: Body(str, required=True), age: Body(int, default=0)):
        pass

    result = ParamResolver.resolve(
        func=handler,
        request=None,
        body_data={'name': 'Alice', 'age': '25'}
    )

    assert result['name'] == 'Alice'
    assert result['age'] == 25
    assert type(result['age']) is int

test_body_param()

@test("Header 参数解析和 alias")
def test_header_param():
    from cullinan.params import Header, ParamResolver

    def handler(self, auth: Header(str, alias='Authorization')):
        pass

    result = ParamResolver.resolve(
        func=handler,
        request=None,
        headers={'Authorization': 'Bearer token123'}
    )

    assert result['auth'] == 'Bearer token123'

test_header_param()

@test("DynamicBody 解析")
def test_dynamic_body():
    from cullinan.params import DynamicBody, ParamResolver

    def handler(self, body: DynamicBody):
        pass

    result = ParamResolver.resolve(
        func=handler,
        request=None,
        body_data={'name': 'Test', 'age': 25, 'nested': {'a': 1}}
    )

    body = result['body']
    assert body.name == 'Test'
    assert body.age == 25
    assert body.nested.a == 1
    assert body.get('missing', 'default') == 'default'

test_dynamic_body()

@test("类型转换 - bool")
def test_type_conversion_bool():
    from cullinan.params import Query, ParamResolver

    def handler(self, val: Query(bool)):
        pass

    for true_val in ['true', 'True', 'TRUE', '1', 'yes', 'on']:
        result = ParamResolver.resolve(func=handler, request=None, query_params={'val': true_val})
        assert result['val'] == True, f"Expected True for '{true_val}'"

    for false_val in ['false', 'False', 'FALSE', '0', 'no', 'off']:
        result = ParamResolver.resolve(func=handler, request=None, query_params={'val': false_val})
        assert result['val'] == False, f"Expected False for '{false_val}'"

test_type_conversion_bool()

@test("参数校验 - ge/le")
def test_param_validation_ge_le():
    from cullinan.params import Body, ParamResolver, ResolveError

    def handler(self, age: Body(int, ge=0, le=150)):
        pass

    # 合法值
    result = ParamResolver.resolve(func=handler, request=None, body_data={'age': '25'})
    assert result['age'] == 25

    # 非法值 - 小于 ge
    try:
        ParamResolver.resolve(func=handler, request=None, body_data={'age': '-1'})
        assert False, "Should raise ResolveError"
    except ResolveError:
        pass

    # 非法值 - 大于 le
    try:
        ParamResolver.resolve(func=handler, request=None, body_data={'age': '200'})
        assert False, "Should raise ResolveError"
    except ResolveError:
        pass

test_param_validation_ge_le()

@test("参数校验 - min_length/max_length")
def test_param_validation_length():
    from cullinan.params import Body, ParamResolver, ResolveError

    def handler(self, name: Body(str, min_length=2, max_length=10)):
        pass

    # 合法值
    result = ParamResolver.resolve(func=handler, request=None, body_data={'name': 'hello'})
    assert result['name'] == 'hello'

    # 非法值 - 太短
    try:
        ParamResolver.resolve(func=handler, request=None, body_data={'name': 'a'})
        assert False, "Should raise ResolveError for min_length"
    except ResolveError:
        pass

test_param_validation_length()


# ============================================================================
# 3. Codec 测试
# ============================================================================
print("\n" + "=" * 70)
print("3. Codec 测试")
print("=" * 70)

@test("JSON Codec 编解码")
def test_json_codec():
    from cullinan.codec import JsonBodyCodec

    codec = JsonBodyCodec()

    # 编码
    data = {'name': 'test', 'value': 123}
    encoded = codec.encode(data)
    assert isinstance(encoded, bytes)

    # 解码
    decoded = codec.decode(encoded)
    assert decoded == data

test_json_codec()

@test("Form Codec 编解码")
def test_form_codec():
    from cullinan.codec import FormBodyCodec

    codec = FormBodyCodec()

    # 编码
    data = {'name': 'test', 'value': '123'}
    encoded = codec.encode(data)
    assert isinstance(encoded, bytes)

    # 解码
    decoded = codec.decode(encoded)
    # Form codec 可能返回不同格式，只检查编解码不抛异常
    assert decoded is not None

test_form_codec()

@test("Codec Registry")
def test_codec_registry():
    from cullinan.codec import get_codec_registry

    registry = get_codec_registry()

    # 使用正确的方法名
    json_codec = registry.get_body_codec('application/json')
    assert json_codec is not None

    form_codec = registry.get_body_codec('application/x-www-form-urlencoded')
    assert form_codec is not None

test_codec_registry()


# ============================================================================
# 4. 控制器相关测试
# ============================================================================
print("\n" + "=" * 70)
print("4. 控制器相关测试")
print("=" * 70)

@test("控制器注册")
def test_controller_registration():
    from cullinan.controller import controller, get_api
    from cullinan.core.pending import PendingRegistry

    PendingRegistry.reset()

    @controller(url='/test')
    class TestController3:
        @get_api(url='/hello')
        def hello(self):
            pass

    # 检查是否在 PendingRegistry 中
    pending = PendingRegistry.get_instance()
    registrations = pending.get_all()

    found = any(r.name == 'TestController3' for r in registrations)
    assert found, "TestController3 should be in PendingRegistry"

test_controller_registration()

@test("URL 解析器")
def test_url_resolver():
    from cullinan.controller.core import url_resolver

    # 简单路径
    url, params = url_resolver('/users/{id}')
    assert 'id' in params, f"Params should contain 'id', got: {params}"
    assert '{id}' not in url, f"URL should have placeholder replaced, got: {url}"

    # 多参数
    url, params = url_resolver('/users/{user_id}/posts/{post_id}')
    assert 'user_id' in params
    assert 'post_id' in params

test_url_resolver()


# ============================================================================
# 5. 性能基准测试
# ============================================================================
print("\n" + "=" * 70)
print("5. 性能基准测试")
print("=" * 70)

@test("ParamResolver 解析性能 (1000次)")
def test_param_resolver_performance():
    from cullinan.params import Path, Query, Body, ParamResolver

    def handler(
        self,
        id: Path(int),
        page: Query(int, default=1),
        size: Query(int, default=10),
        name: Body(str, required=True),
        age: Body(int, default=0),
    ):
        pass

    start = time.perf_counter()
    iterations = 1000

    for _ in range(iterations):
        ParamResolver.resolve(
            func=handler,
            request=None,
            url_params={'id': '123'},
            query_params={'page': '5', 'size': '20'},
            body_data={'name': 'Test', 'age': '25'}
        )

    elapsed = (time.perf_counter() - start) * 1000
    avg = elapsed / iterations

    print(f"\n    Total: {elapsed:.2f}ms, Avg: {avg:.4f}ms/call", end="")

    # 性能阈值：每次解析应小于 0.5ms
    assert avg < 0.5, f"Performance too slow: {avg:.4f}ms/call"

test_param_resolver_performance()

@test("DynamicBody 访问性能 (10000次)")
def test_dynamic_body_performance():
    from cullinan.params import DynamicBody

    body = DynamicBody({
        'name': 'Test',
        'age': 25,
        'nested': {'a': 1, 'b': 2},
        'items': [1, 2, 3, 4, 5]
    })

    start = time.perf_counter()
    iterations = 10000

    for _ in range(iterations):
        _ = body.name
        _ = body.age
        _ = body.nested.a
        _ = body.get('missing', 'default')

    elapsed = (time.perf_counter() - start) * 1000
    avg = elapsed / iterations

    print(f"\n    Total: {elapsed:.2f}ms, Avg: {avg:.6f}ms/call", end="")

    # 性能阈值：每次访问应小于 0.01ms
    assert avg < 0.01, f"Performance too slow: {avg:.6f}ms/call"

test_dynamic_body_performance()

@test("类型转换性能 (5000次)")
def test_type_conversion_performance():
    from cullinan.params.converter import TypeConverter

    start = time.perf_counter()
    iterations = 5000

    for _ in range(iterations):
        TypeConverter.convert('123', int)
        TypeConverter.convert('3.14', float)
        TypeConverter.convert('true', bool)
        TypeConverter.convert('hello', str)

    elapsed = (time.perf_counter() - start) * 1000
    avg = elapsed / (iterations * 4)

    print(f"\n    Total: {elapsed:.2f}ms, Avg: {avg:.6f}ms/call", end="")

    # 性能阈值
    assert avg < 0.01, f"Performance too slow: {avg:.6f}ms/call"

test_type_conversion_performance()


# ============================================================================
# 6. 回归测试
# ============================================================================
print("\n" + "=" * 70)
print("6. 回归测试")
print("=" * 70)

@test("传统参数方式仍然支持")
def test_legacy_params():
    from cullinan.params import ParamResolver

    # 传统方式：参数名为 url_params, query_params 等
    def legacy_handler(self, url_params, query_params, body_params):
        pass

    config = ParamResolver.analyze_params(legacy_handler)

    # 传统参数应该被识别为 unknown source
    assert config['url_params']['source'] == 'unknown'
    assert config['query_params']['source'] == 'unknown'
    assert config['body_params']['source'] == 'unknown'

test_legacy_params()

@test("混合参数方式")
def test_mixed_params():
    from cullinan.params import Path, Query, ParamResolver

    # 混合方式：部分使用新参数，部分使用传统
    def mixed_handler(self, id: Path(int), url_params):
        pass

    config = ParamResolver.analyze_params(mixed_handler)

    assert config['id']['source'] == 'path'
    assert config['url_params']['source'] == 'unknown'

test_mixed_params()

@test("服务生命周期")
def test_service_lifecycle():
    from cullinan.service import service, Service
    from cullinan.core import ApplicationContext
    from cullinan.core.pending import PendingRegistry

    PendingRegistry.reset()

    lifecycle_events = []

    @service
    class LifecycleService(Service):
        def on_init(self):
            lifecycle_events.append('on_init')

        def on_startup(self):
            lifecycle_events.append('on_startup')

    ctx = ApplicationContext()
    ctx.refresh()

    # on_init 应该在 refresh 时被调用
    svc = ctx.get('LifecycleService')
    assert svc is not None
    # 注意：on_init 可能在实例化时调用，取决于实现

test_service_lifecycle()


# ============================================================================
# 测试总结
# ============================================================================
print("\n" + "=" * 70)
print("测试总结")
print("=" * 70)

total = test_results['passed'] + test_results['failed']
print(f"\n总测试数: {total}")
print(f"通过: {test_results['passed']}")
print(f"失败: {test_results['failed']}")

if test_results['errors']:
    print("\n失败详情:")
    for name, error in test_results['errors']:
        print(f"\n  {name}:")
        print(f"    {error[:200]}...")

print("\n" + "=" * 70)
if test_results['failed'] == 0:
    print("所有测试通过!")
else:
    print(f"有 {test_results['failed']} 个测试失败!")
    sys.exit(1)




# -*- coding: utf-8 -*-
"""Cullinan Codec Registry

Codec 注册表，管理所有 BodyCodec 和 ResponseCodec。

Author: Plumeink
"""

from typing import Any, Dict, List, Optional, Tuple, Type

from .base import BodyCodec, ResponseCodec
from .errors import DecodeError, EncodeError


class CodecRegistry:
    """Codec 注册表

    管理所有 BodyCodec 和 ResponseCodec 的注册和选择。

    Example:
        registry = get_codec_registry()

        # 解码请求体
        data = registry.decode_body(body, 'application/json')

        # 编码响应
        encoded, content_type = registry.encode_response(data, 'application/json')

        # 注册自定义 Codec
        registry.register_body_codec(XmlBodyCodec)
    """

    def __init__(self):
        self._body_codecs: List[Type[BodyCodec]] = []
        self._response_codecs: List[Type[ResponseCodec]] = []

    def register_body_codec(self, codec_class: Type[BodyCodec]) -> None:
        """注册请求体 Codec

        Args:
            codec_class: BodyCodec 子类
        """
        if codec_class not in self._body_codecs:
            self._body_codecs.append(codec_class)
            # 按优先级排序 (数字越小越优先)
            self._body_codecs.sort(key=lambda c: c.priority)

    def register_response_codec(self, codec_class: Type[ResponseCodec]) -> None:
        """注册响应 Codec

        Args:
            codec_class: ResponseCodec 子类
        """
        if codec_class not in self._response_codecs:
            self._response_codecs.append(codec_class)
            self._response_codecs.sort(key=lambda c: c.priority)

    def unregister_body_codec(self, codec_class: Type[BodyCodec]) -> None:
        """取消注册请求体 Codec"""
        if codec_class in self._body_codecs:
            self._body_codecs.remove(codec_class)

    def unregister_response_codec(self, codec_class: Type[ResponseCodec]) -> None:
        """取消注册响应 Codec"""
        if codec_class in self._response_codecs:
            self._response_codecs.remove(codec_class)

    def get_body_codec(self, content_type: str) -> Optional[BodyCodec]:
        """根据 Content-Type 获取请求体 Codec 实例

        Args:
            content_type: HTTP Content-Type 头

        Returns:
            Codec 实例，无匹配返回 None
        """
        for codec_class in self._body_codecs:
            if codec_class.supports(content_type):
                return codec_class()
        return None

    def get_response_codec(self, accept: str = '*/*') -> Optional[ResponseCodec]:
        """根据 Accept 获取响应 Codec 实例

        Args:
            accept: HTTP Accept 头

        Returns:
            Codec 实例，无匹配返回 None
        """
        for codec_class in self._response_codecs:
            if codec_class.supports_accept(accept):
                return codec_class()
        return None

    def decode_body(
        self,
        body: bytes,
        content_type: str,
        charset: str = 'utf-8'
    ) -> Dict[str, Any]:
        """解码请求体

        Args:
            body: 原始请求体字节
            content_type: Content-Type 头
            charset: 字符编码

        Returns:
            解码后的字典

        Raises:
            DecodeError: 解码失败
        """
        codec = self.get_body_codec(content_type)
        if codec:
            return codec.decode(body, charset)

        # 默认尝试 JSON
        from .json_codec import JsonBodyCodec
        return JsonBodyCodec().decode(body, charset)

    def encode_response(
        self,
        data: Any,
        accept: str = '*/*',
        charset: str = 'utf-8'
    ) -> Tuple[bytes, str]:
        """编码响应

        Args:
            data: 响应数据
            accept: Accept 头
            charset: 字符编码

        Returns:
            (编码后的字节, Content-Type)

        Raises:
            EncodeError: 编码失败
        """
        codec = self.get_response_codec(accept)
        if codec:
            return codec.encode(data, charset), codec.get_content_type(charset)

        # 默认 JSON
        from .json_codec import JsonResponseCodec
        default_codec = JsonResponseCodec()
        return default_codec.encode(data, charset), default_codec.get_content_type(charset)

    def list_body_codecs(self) -> List[Type[BodyCodec]]:
        """列出所有注册的请求体 Codec"""
        return list(self._body_codecs)

    def list_response_codecs(self) -> List[Type[ResponseCodec]]:
        """列出所有注册的响应 Codec"""
        return list(self._response_codecs)


# 全局注册表实例
_codec_registry: Optional[CodecRegistry] = None


def get_codec_registry() -> CodecRegistry:
    """获取全局 Codec 注册表

    Returns:
        CodecRegistry 单例
    """
    global _codec_registry
    if _codec_registry is None:
        _codec_registry = CodecRegistry()
        # 注册默认 Codec
        from .json_codec import JsonBodyCodec, JsonResponseCodec
        from .form_codec import FormBodyCodec
        _codec_registry.register_body_codec(JsonBodyCodec)
        _codec_registry.register_body_codec(FormBodyCodec)
        _codec_registry.register_response_codec(JsonResponseCodec)
    return _codec_registry


def reset_codec_registry() -> None:
    """重置全局注册表 (测试用)"""
    global _codec_registry
    _codec_registry = None




# -*- coding: utf-8 -*-
"""Cullinan Parameter Resolver

参数解析编排器，协调各层完成参数解析。

Author: Plumeink
"""

import inspect
import dataclasses
from typing import Any, Callable, Dict, List, Optional, Tuple, Type, get_type_hints

from .base import Param, UNSET
from .types import Path, Query, Body, Header, File, RawBody
from .converter import TypeConverter, ConversionError
from .auto import Auto, AutoType
from .dynamic import DynamicBody
from .validator import ParamValidator, ValidationError
from .model import ModelResolver, ModelError
from .file_info import FileInfo, FileList
from .model_handlers import get_model_handler_registry, ModelHandlerError


class ResolveError(Exception):
    """参数解析错误

    Attributes:
        message: 错误消息
        errors: 错误详情列表
    """

    def __init__(self, message: str, errors: list = None):
        super().__init__(message)
        self.message = message
        self.errors = errors or []

    def to_dict(self) -> dict:
        """转换为字典格式"""
        return {
            'message': self.message,
            'errors': self.errors,
        }


class ParamResolver:
    """参数解析编排器

    协调传输层、转换层、数据层完成参数解析。

    Features:
    - 从函数签名推导参数配置
    - 支持传统装饰器配置
    - 自动类型转换
    - 参数校验
    - dataclass / DynamicBody 支持

    Example:
        @post_api(url="/users")
        async def create_user(
            self,
            name: Body(str, required=True),
            age: Body(int, default=0, ge=0),
        ):
            pass

        # ParamResolver 会自动解析 name 和 age 参数
    """

    # 签名缓存
    _signature_cache: Dict[Callable, inspect.Signature] = {}

    @classmethod
    def _get_header_value(cls, headers: Dict[str, str], key: str) -> Optional[str]:
        """Get header value with case-insensitive matching

        HTTP headers are case-insensitive per RFC 7230.

        Args:
            headers: Header dictionary
            key: Header name to look up

        Returns:
            Header value or None
        """
        if not headers or not key:
            return None

        # Try exact match first
        if key in headers:
            return headers[key]

        # Try case-insensitive match
        key_lower = key.lower()
        for header_key, header_value in headers.items():
            if header_key.lower() == key_lower:
                return header_value

        return None

    @classmethod
    def get_signature(cls, func: Callable) -> inspect.Signature:
        """获取函数签名 (带缓存)

        Args:
            func: 函数

        Returns:
            函数签名
        """
        if func not in cls._signature_cache:
            cls._signature_cache[func] = inspect.signature(func)
        return cls._signature_cache[func]

    @classmethod
    def analyze_params(cls, func: Callable) -> Dict[str, dict]:
        """分析函数参数配置

        Args:
            func: 控制器方法

        Returns:
            参数配置字典 {param_name: {source, type, param_spec, ...}}
        """
        sig = cls.get_signature(func)
        type_hints = {}
        try:
            type_hints = get_type_hints(func)
        except Exception:
            pass

        params_config = {}

        for name, param in sig.parameters.items():
            # 跳过 self 参数
            if name == 'self':
                continue

            config = {
                'name': name,
                'source': 'unknown',
                'type': str,
                'required': True,
                'default': UNSET,
                'param_spec': None,
            }

            # 检查类型注解
            annotation = type_hints.get(name, param.annotation)

            if annotation is inspect.Parameter.empty:
                annotation = None

            # 优先检查默认值是否是 Param 实例（支持简化语法）
            # 例如: sign: str = Header(alias="X-Hub-Signature-256")
            if param.default is not inspect.Parameter.empty and isinstance(param.default, Param):
                param_spec = param.default
                config['source'] = param_spec.source
                # 如果 Param 没有指定类型，从注解获取
                if param_spec.type_ is str and annotation is not None and annotation is not str:
                    config['type'] = annotation
                else:
                    config['type'] = param_spec.type_
                config['required'] = param_spec.required
                config['default'] = param_spec.default
                config['param_spec'] = param_spec
                # 使用参数名作为 name（如果未指定）
                if param_spec.name is None:
                    param_spec.name = name

            # 检查默认值是否是 DynamicBody 实例
            # 例如: body: DynamicBody = DynamicBody()
            elif param.default is not inspect.Parameter.empty and isinstance(param.default, DynamicBody):
                config['source'] = 'body'
                config['type'] = DynamicBody
                config['required'] = False
                config['default'] = param.default

            # 检查是否是 Param 实例（类型注解方式）
            elif isinstance(annotation, Param):
                param_spec = annotation
                config['source'] = param_spec.source
                config['type'] = param_spec.type_
                config['required'] = param_spec.required
                config['default'] = param_spec.default
                config['param_spec'] = param_spec
                # 使用参数名作为 name（如果未指定）
                if param_spec.name is None:
                    param_spec.name = name

            # 检查是否是 DynamicBody
            elif annotation is DynamicBody:
                config['source'] = 'body'
                config['type'] = DynamicBody
                config['required'] = False

            # 检查是否是 RawBody（类本身，不需要括号）
            elif annotation is RawBody:
                config['source'] = 'raw_body'
                config['type'] = bytes
                config['required'] = False

            # 检查是否是可处理的模型类型（通过注册表）
            elif get_model_handler_registry().can_handle(annotation):
                handler = get_model_handler_registry().get_handler(annotation)
                config['source'] = handler.get_source()
                config['type'] = annotation
                config['required'] = handler.is_required_by_default()
                config['model_handler'] = handler

            # 检查是否是 AutoType
            elif annotation is AutoType:
                config['source'] = 'auto'
                config['type'] = AutoType

            # 普通类型注解（默认作为 Query 参数）
            elif annotation in (str, int, float, bool):
                config['source'] = 'query'
                config['type'] = annotation

            # 其他类型注解
            elif annotation is not None:
                config['type'] = annotation

            # 检查默认值（非 Param 实例的普通默认值）
            if param.default is not inspect.Parameter.empty and not isinstance(param.default, Param):
                config['default'] = param.default
                config['required'] = False

            params_config[name] = config

        return params_config

    @classmethod
    def resolve(
        cls,
        func: Callable,
        request: Any,
        url_params: Dict[str, Any] = None,
        query_params: Dict[str, Any] = None,
        body_data: Dict[str, Any] = None,
        headers: Dict[str, str] = None,
        files: Dict[str, Any] = None,
    ) -> Dict[str, Any]:
        """解析请求参数

        Args:
            func: 控制器方法
            request: Tornado request 对象
            url_params: URL 路径参数
            query_params: 查询参数
            body_data: 请求体数据 (已解码)
            headers: 请求头
            files: 上传的文件

        Returns:
            解析后的参数字典 {param_name: value}

        Raises:
            ResolveError: 解析失败
        """
        url_params = url_params or {}
        query_params = query_params or {}
        body_data = body_data or {}
        headers = headers or {}
        files = files or {}

        # 分析参数配置
        params_config = cls.analyze_params(func)

        result = {}
        errors = []

        for name, config in params_config.items():
            try:
                value = cls._resolve_param(
                    name, config,
                    url_params, query_params, body_data, headers, files, request
                )
                result[name] = value
            except (ConversionError, ValidationError, ModelError) as e:
                errors.append({
                    'param': name,
                    'error': str(e),
                    'type': type(e).__name__,
                })

        if errors:
            raise ResolveError(
                f"Failed to resolve {len(errors)} parameter(s)",
                errors=errors
            )

        return result

    @classmethod
    def _resolve_param(
        cls,
        name: str,
        config: dict,
        url_params: Dict[str, Any],
        query_params: Dict[str, Any],
        body_data: Dict[str, Any],
        headers: Dict[str, str],
        files: Dict[str, Any],
        request: Any = None,
    ) -> Any:
        """解析单个参数

        Args:
            name: 参数名
            config: 参数配置
            各种来源数据...
            request: 原始请求对象（用于 RawBody）

        Returns:
            解析后的值
        """
        source = config['source']
        target_type = config['type']
        required = config['required']
        default = config['default']
        param_spec = config.get('param_spec')

        # 获取别名
        alias = name
        if param_spec and param_spec.alias:
            alias = param_spec.alias

        # 根据来源获取原始值
        raw_value = None

        if source == 'path':
            raw_value = url_params.get(alias) or url_params.get(name)
        elif source == 'query':
            raw_value = query_params.get(alias) or query_params.get(name)
        elif source == 'body':
            # 特殊处理 DynamicBody 和模型类型
            if target_type is DynamicBody:
                return DynamicBody(body_data)
            elif config.get('model_handler'):
                # 使用注册的模型处理器
                return config['model_handler'].resolve(target_type, body_data)
            else:
                raw_value = body_data.get(alias) or body_data.get(name)
        elif source == 'header':
            # HTTP headers are case-insensitive, use lowercase matching
            raw_value = cls._get_header_value(headers, alias) or cls._get_header_value(headers, name)
        elif source == 'raw_body':
            # 返回原始二进制请求体
            if request is not None and hasattr(request, 'body'):
                return request.body if request.body else b''
            return b''
        elif source == 'file':
            raw_value = files.get(alias) or files.get(name)
            # 处理文件参数
            if raw_value is not None:
                return cls._resolve_file_param(raw_value, param_spec, name)
        elif source == 'auto':
            # Auto 类型：依次查找各来源
            raw_value = (
                url_params.get(name) or
                query_params.get(name) or
                body_data.get(name)
            )
        else:
            # 未知来源：尝试从多处获取
            raw_value = (
                url_params.get(name) or
                query_params.get(name) or
                body_data.get(name)
            )

        # 处理 None 值
        if raw_value is None:
            if default is not UNSET:
                return default
            if required:
                raise ValidationError(
                    f"Parameter '{name}' is required",
                    param_name=name,
                    constraint='required'
                )
            return None

        # 类型转换
        if target_type is AutoType:
            converted = Auto.infer(raw_value)
        elif target_type is DynamicBody:
            if isinstance(raw_value, dict):
                converted = DynamicBody(raw_value)
            else:
                converted = raw_value
        elif config.get('model_handler'):
            # 使用注册的模型处理器
            if isinstance(raw_value, dict):
                converted = config['model_handler'].resolve(target_type, raw_value)
            else:
                converted = raw_value
        else:
            converted = TypeConverter.convert(raw_value, target_type)

        # 参数校验
        if param_spec:
            ParamValidator.validate_param(param_spec, converted, name)

        return converted

    @classmethod
    def clear_cache(cls) -> None:
        """清空签名缓存"""
        cls._signature_cache.clear()

    @classmethod
    def _resolve_file_param(
        cls,
        raw_value: Any,
        param_spec: File,
        name: str,
    ) -> Any:
        """解析文件参数

        Args:
            raw_value: 原始文件数据 (来自 Tornado)
            param_spec: File 参数规格
            name: 参数名

        Returns:
            FileInfo 或 FileList
        """
        # 处理列表格式 (Tornado 的 request.files 返回列表)
        if isinstance(raw_value, list):
            file_list = []
            for item in raw_value:
                if isinstance(item, dict):
                    file_info = FileInfo.from_tornado_file(item)
                elif isinstance(item, FileInfo):
                    file_info = item
                else:
                    # 尝试作为字典处理
                    file_info = FileInfo(
                        filename=getattr(item, 'filename', 'unknown'),
                        body=getattr(item, 'body', b''),
                        content_type=getattr(item, 'content_type', None),
                    )
                file_list.append(file_info)

            if param_spec and param_spec.multiple:
                # 多文件模式
                result = FileList(file_list)
                if param_spec:
                    param_spec.validate_file_list(result)
                return result
            elif file_list:
                # 单文件模式，取第一个
                result = file_list[0]
                if param_spec:
                    param_spec.validate_file(result)
                return result
            else:
                return None

        # 处理单个文件
        if isinstance(raw_value, dict):
            file_info = FileInfo.from_tornado_file(raw_value)
        elif isinstance(raw_value, FileInfo):
            file_info = raw_value
        else:
            file_info = FileInfo(
                filename=getattr(raw_value, 'filename', 'unknown'),
                body=getattr(raw_value, 'body', b''),
                content_type=getattr(raw_value, 'content_type', None),
            )

        if param_spec:
            param_spec.validate_file(file_info)

        return file_info




# -*- coding: utf-8 -*-
"""全面测试：所有参数类型解析验证

Author: Plumeink
"""

import sys
import os
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from cullinan.params import Path, Query, Body, Header, File, DynamicBody, ParamResolver, AutoType
from cullinan.params.validator import ValidationError

print("=" * 70)
print("全面参数类型测试")
print("=" * 70)

# ============================================================================
# 测试1: Path 参数
# ============================================================================
print("\n1. Path 参数测试")
print("-" * 40)

def test_path(self, id: Path(int), name: Path(str)):
    pass

config = ParamResolver.analyze_params(test_path)
print(f"   参数配置: {list(config.keys())}")
for name, cfg in config.items():
    print(f"   - {name}: source={cfg['source']}, type={cfg['type'].__name__}, required={cfg['required']}")

# 测试解析
result = ParamResolver.resolve(
    func=test_path,
    request=None,
    url_params={'id': '123', 'name': 'test'}
)
print(f"   解析结果: {result}")
print(f"   id 类型: {type(result['id']).__name__}, 值: {result['id']}")
assert result['id'] == 123, "Path(int) 转换失败"
assert result['name'] == 'test', "Path(str) 转换失败"
print("   ✅ Path 参数测试通过")

# ============================================================================
# 测试2: Query 参数
# ============================================================================
print("\n2. Query 参数测试")
print("-" * 40)

def test_query(
    self,
    page: Query(int, default=1, ge=1),
    size: Query(int, default=10, ge=1, le=100),
    q: Query(str, required=False),
    active: Query(bool, default=True),
):
    pass

config = ParamResolver.analyze_params(test_query)
print(f"   参数配置: {list(config.keys())}")

# 测试解析 - 有值
result = ParamResolver.resolve(
    func=test_query,
    request=None,
    query_params={'page': '2', 'size': '20', 'q': 'hello', 'active': 'false'}
)
print(f"   解析结果(有值): {result}")
assert result['page'] == 2, "Query(int) 转换失败"
assert result['size'] == 20, "Query(int) 转换失败"
assert result['q'] == 'hello', "Query(str) 转换失败"
assert result['active'] == False, "Query(bool) 转换失败"

# 测试解析 - 使用默认值
result = ParamResolver.resolve(
    func=test_query,
    request=None,
    query_params={}
)
print(f"   解析结果(默认): {result}")
assert result['page'] == 1, "Query 默认值失败"
assert result['size'] == 10, "Query 默认值失败"
assert result['q'] is None, "Query optional 应为 None"
assert result['active'] == True, "Query(bool) 默认值失败"
print("   ✅ Query 参数测试通过")

# ============================================================================
# 测试3: Body 参数
# ============================================================================
print("\n3. Body 参数测试")
print("-" * 40)

def test_body(
    self,
    name: Body(str, required=True),
    age: Body(int, default=0, ge=0, le=150),
    email: Body(str, required=False),
):
    pass

config = ParamResolver.analyze_params(test_body)
print(f"   参数配置: {list(config.keys())}")

# 测试解析 - 有值
result = ParamResolver.resolve(
    func=test_body,
    request=None,
    body_data={'name': 'John', 'age': '25', 'email': 'john@example.com'}
)
print(f"   解析结果: {result}")
assert result['name'] == 'John', "Body(str) 转换失败"
assert result['age'] == 25, "Body(int) 转换失败"
assert result['email'] == 'john@example.com', "Body(str) 转换失败"

# 测试解析 - 使用默认值
result = ParamResolver.resolve(
    func=test_body,
    request=None,
    body_data={'name': 'Jane'}
)
print(f"   解析结果(默认): {result}")
assert result['name'] == 'Jane', "Body(str) 转换失败"
assert result['age'] == 0, "Body 默认值失败"
assert result['email'] is None, "Body optional 应为 None"
print("   ✅ Body 参数测试通过")

# ============================================================================
# 测试4: Header 参数
# ============================================================================
print("\n4. Header 参数测试")
print("-" * 40)

def test_header(
    self,
    auth: Header(str, alias='Authorization', required=True),
    request_id: Header(str, alias='X-Request-ID', required=False),
):
    pass

config = ParamResolver.analyze_params(test_header)
print(f"   参数配置: {list(config.keys())}")
for name, cfg in config.items():
    alias = cfg.get('param_spec').alias if cfg.get('param_spec') else None
    print(f"   - {name}: source={cfg['source']}, alias={alias}")

# 测试解析
result = ParamResolver.resolve(
    func=test_header,
    request=None,
    headers={'Authorization': 'Bearer token123', 'X-Request-ID': 'req-456'}
)
print(f"   解析结果: {result}")
assert result['auth'] == 'Bearer token123', "Header 解析失败"
assert result['request_id'] == 'req-456', "Header 解析失败"

# 测试解析 - 只有必填
result = ParamResolver.resolve(
    func=test_header,
    request=None,
    headers={'Authorization': 'Bearer token789'}
)
print(f"   解析结果(部分): {result}")
assert result['auth'] == 'Bearer token789', "Header 解析失败"
assert result['request_id'] is None, "Header optional 应为 None"
print("   ✅ Header 参数测试通过")

# ============================================================================
# 测试5: DynamicBody
# ============================================================================
print("\n5. DynamicBody 测试")
print("-" * 40)

def test_dynamic_body(self, body: DynamicBody):
    pass

config = ParamResolver.analyze_params(test_dynamic_body)
print(f"   参数配置: {list(config.keys())}")
for name, cfg in config.items():
    print(f"   - {name}: source={cfg['source']}, type={cfg['type'].__name__}")

# 测试解析
result = ParamResolver.resolve(
    func=test_dynamic_body,
    request=None,
    body_data={'name': 'Test', 'nested': {'a': 1, 'b': 2}, 'tags': [1, 2, 3]}
)
print(f"   解析结果: {result}")
body = result['body']
print(f"   body.name = {body.name}")
print(f"   body.nested = {body.nested}")
print(f"   body.tags = {body.tags}")
print(f"   body.get('missing', 'default') = {body.get('missing', 'default')}")
assert body.name == 'Test', "DynamicBody.name 失败"
assert body.nested.a == 1, "DynamicBody 嵌套访问失败"
assert body.tags == [1, 2, 3], "DynamicBody 列表访问失败"
# 注意：'items', 'keys', 'values', 'get' 等字典方法名会与属性名冲突
# 使用 body['items'] 或 body.to_dict()['items'] 访问这些特殊名称
print("   ✅ DynamicBody 测试通过")

# ============================================================================
# 测试6: 校验测试
# ============================================================================
print("\n6. 参数校验测试")
print("-" * 40)

def test_validation(
    self,
    age: Body(int, ge=0, le=150),
    name: Body(str, min_length=2, max_length=50),
):
    pass

# 测试校验通过
result = ParamResolver.resolve(
    func=test_validation,
    request=None,
    body_data={'age': '25', 'name': 'John'}
)
print(f"   校验通过: {result}")

# 测试校验失败 - ge
try:
    result = ParamResolver.resolve(
        func=test_validation,
        request=None,
        body_data={'age': '-1', 'name': 'John'}
    )
    print("   ❌ ge 校验应该失败")
except Exception as e:
    print(f"   ✅ ge 校验正确失败: {e}")

# 测试校验失败 - min_length
try:
    result = ParamResolver.resolve(
        func=test_validation,
        request=None,
        body_data={'age': '25', 'name': 'J'}
    )
    print("   ❌ min_length 校验应该失败")
except Exception as e:
    print(f"   ✅ min_length 校验正确失败: {e}")

print("   ✅ 参数校验测试通过")

# ============================================================================
# 测试7: 混合参数测试
# ============================================================================
print("\n7. 混合参数测试")
print("-" * 40)

def test_mixed(
    self,
    id: Path(int),
    page: Query(int, default=1),
    name: Body(str, required=True),
    auth: Header(str, alias='Authorization'),
):
    pass

config = ParamResolver.analyze_params(test_mixed)
print(f"   参数配置: {list(config.keys())}")
for name, cfg in config.items():
    print(f"   - {name}: source={cfg['source']}")

result = ParamResolver.resolve(
    func=test_mixed,
    request=None,
    url_params={'id': '42'},
    query_params={'page': '3'},
    body_data={'name': 'Alice'},
    headers={'Authorization': 'Bearer xxx'}
)
print(f"   解析结果: {result}")
assert result['id'] == 42, "混合 Path 失败"
assert result['page'] == 3, "混合 Query 失败"
assert result['name'] == 'Alice', "混合 Body 失败"
assert result['auth'] == 'Bearer xxx', "混合 Header 失败"
print("   ✅ 混合参数测试通过")

# ============================================================================
# 测试8: 类型转换测试
# ============================================================================
print("\n8. 类型转换测试")
print("-" * 40)

def test_types(
    self,
    int_val: Query(int),
    float_val: Query(float),
    bool_val: Query(bool),
    str_val: Query(str),
):
    pass

# 测试各种类型转换
result = ParamResolver.resolve(
    func=test_types,
    request=None,
    query_params={
        'int_val': '123',
        'float_val': '3.14',
        'bool_val': 'true',
        'str_val': 'hello'
    }
)
print(f"   解析结果: {result}")
assert result['int_val'] == 123 and type(result['int_val']) is int, "int 转换失败"
assert result['float_val'] == 3.14 and type(result['float_val']) is float, "float 转换失败"
assert result['bool_val'] == True and type(result['bool_val']) is bool, "bool 转换失败"
assert result['str_val'] == 'hello' and type(result['str_val']) is str, "str 转换失败"

# bool 多种格式测试
def test_bool_formats(self, val: Query(bool)):
    pass

for true_val in ['true', 'True', 'TRUE', '1', 'yes', 'Yes', 'on']:
    result = ParamResolver.resolve(
        func=test_bool_formats, request=None, query_params={'val': true_val}
    )
    assert result['val'] == True, f"bool 转换 {true_val} 失败"

for false_val in ['false', 'False', 'FALSE', '0', 'no', 'No', 'off']:
    result = ParamResolver.resolve(
        func=test_bool_formats, request=None, query_params={'val': false_val}
    )
    assert result['val'] == False, f"bool 转换 {false_val} 失败"

print("   ✅ 类型转换测试通过")

print("\n" + "=" * 70)
print("所有参数类型测试完成！")
print("=" * 70)




# -*- coding: utf-8 -*-
"""Cullinan Response Decorators

响应模型装饰器，用于定义 API 响应模式。

Author: Plumeink
"""

from typing import Any, Type, List, Dict, Callable, Optional, Union
from functools import wraps
import dataclasses
import json


class ResponseModel:
    """响应模型定义

    Attributes:
        model: 响应模型类 (dataclass 或普通类)
        status_code: HTTP 状态码
        description: 响应描述
        content_type: 响应内容类型
    """

    __slots__ = ('model', 'status_code', 'description', 'content_type', 'headers')

    def __init__(
        self,
        model: Type = None,
        status_code: int = 200,
        description: str = '',
        content_type: str = 'application/json',
        headers: Dict[str, str] = None,
    ):
        self.model = model
        self.status_code = status_code
        self.description = description
        self.content_type = content_type
        self.headers = headers or {}

    def __repr__(self) -> str:
        return (
            f"ResponseModel(model={self.model.__name__ if self.model else None}, "
            f"status_code={self.status_code})"
        )


def Response(
    model: Type = None,
    status_code: int = 200,
    description: str = '',
    content_type: str = 'application/json',
    headers: Dict[str, str] = None,
):
    """响应模型装饰器

    用于定义 API 端点的响应模式，支持多个响应状态。

    Args:
        model: 响应模型类
        status_code: HTTP 状态码
        description: 响应描述
        content_type: 响应内容类型
        headers: 响应头

    Example:
        from dataclasses import dataclass
        from cullinan.params import Response

        @dataclass
        class UserResponse:
            id: int
            name: str
            email: str

        @dataclass
        class ErrorResponse:
            message: str
            code: int = 0

        @controller(url='/api/users')
        class UserController:
            @get_api(url='/{id}')
            @Response(model=UserResponse, status_code=200, description="User found")
            @Response(model=ErrorResponse, status_code=404, description="User not found")
            async def get_user(self, id: Path(int)):
                user = self.user_service.get(id)
                if not user:
                    return ErrorResponse(message="User not found"), 404
                return UserResponse(id=user.id, name=user.name, email=user.email)
    """
    response_model = ResponseModel(
        model=model,
        status_code=status_code,
        description=description,
        content_type=content_type,
        headers=headers,
    )

    def decorator(func: Callable) -> Callable:
        # 获取或创建响应模型列表
        if not hasattr(func, '_response_models'):
            func._response_models = []

        func._response_models.append(response_model)

        @wraps(func)
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)

        # 保持响应模型列表
        wrapper._response_models = func._response_models

        return wrapper

    return decorator


def get_response_models(func: Callable) -> List[ResponseModel]:
    """获取函数的响应模型列表

    Args:
        func: 被装饰的函数

    Returns:
        ResponseModel 列表
    """
    return getattr(func, '_response_models', [])


class ResponseSerializer:
    """响应序列化器

    将响应数据序列化为 JSON 格式。
    支持 dataclass、dict、list 和基本类型。
    """

    @classmethod
    def serialize(cls, data: Any) -> Any:
        """序列化响应数据

        Args:
            data: 响应数据

        Returns:
            可 JSON 序列化的数据
        """
        if data is None:
            return None

        # dataclass
        if dataclasses.is_dataclass(data) and not isinstance(data, type):
            return cls._serialize_dataclass(data)

        # 字典
        if isinstance(data, dict):
            return {k: cls.serialize(v) for k, v in data.items()}

        # 列表/元组
        if isinstance(data, (list, tuple)):
            return [cls.serialize(item) for item in data]

        # 基本类型
        if isinstance(data, (str, int, float, bool)):
            return data

        # bytes
        if isinstance(data, bytes):
            return data.decode('utf-8', errors='replace')

        # 有 to_dict 方法
        if hasattr(data, 'to_dict') and callable(data.to_dict):
            return cls.serialize(data.to_dict())

        # 有 __dict__ 属性
        if hasattr(data, '__dict__'):
            return {k: cls.serialize(v) for k, v in data.__dict__.items()
                    if not k.startswith('_')}

        # 其他情况转字符串
        return str(data)

    @classmethod
    def _serialize_dataclass(cls, instance) -> dict:
        """序列化 dataclass 实例

        Args:
            instance: dataclass 实例

        Returns:
            字典
        """
        result = {}
        for field in dataclasses.fields(instance):
            value = getattr(instance, field.name)
            result[field.name] = cls.serialize(value)
        return result

    @classmethod
    def to_json(cls, data: Any, **kwargs) -> str:
        """序列化为 JSON 字符串

        Args:
            data: 响应数据
            **kwargs: json.dumps 的参数

        Returns:
            JSON 字符串
        """
        serialized = cls.serialize(data)
        return json.dumps(serialized, ensure_ascii=False, **kwargs)


def serialize_response(data: Any) -> Any:
    """序列化响应数据的便捷函数

    Args:
        data: 响应数据

    Returns:
        可 JSON 序列化的数据
    """
    return ResponseSerializer.serialize(data)




# -*- coding: utf-8 -*-
"""测试新参数系统集成

Author: Plumeink
"""

import sys
import os
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from cullinan.params import DynamicBody, Path, Query, Body, ParamResolver

# 模拟控制器方法
def test_legacy_style(self, url_params, query_params, body_params):
    """传统方式"""
    pass

def test_new_style_dynamic_body(self, body: DynamicBody):
    """新方式：DynamicBody"""
    pass

def test_new_style_typed_params(self, id: Path(int), name: Query(str, default='')):
    """新方式：类型化参数"""
    pass

def test_mixed_style(self, body_params, name: Body(str, required=True)):
    """混合方式"""
    pass

def test_no_params(self):
    """无参数"""
    pass

# 测试分析
print("=== ParamResolver 参数分析测试 ===\n")

# 测试1：传统方式
print("1. 传统方式 (url_params, query_params, body_params):")
config = ParamResolver.analyze_params(test_legacy_style)
for name, cfg in config.items():
    print(f"   - {name}: source={cfg['source']}, type={cfg['type']}")
print()

# 测试2：新方式 DynamicBody
print("2. 新方式 DynamicBody (body: DynamicBody):")
config = ParamResolver.analyze_params(test_new_style_dynamic_body)
for name, cfg in config.items():
    print(f"   - {name}: source={cfg['source']}, type={cfg['type']}")
print()

# 测试3：新方式类型化参数
print("3. 新方式类型化参数 (id: Path(int), name: Query(str)):")
config = ParamResolver.analyze_params(test_new_style_typed_params)
for name, cfg in config.items():
    print(f"   - {name}: source={cfg['source']}, type={cfg['type']}, param_spec={cfg.get('param_spec')}")
print()

# 测试4：混合方式
print("4. 混合方式 (body_params, name: Body(str)):")
config = ParamResolver.analyze_params(test_mixed_style)
for name, cfg in config.items():
    print(f"   - {name}: source={cfg['source']}, type={cfg['type']}")
print()

# 测试5：无参数
print("5. 无参数 (self):")
config = ParamResolver.analyze_params(test_no_params)
print(f"   参数数量: {len(config)}")
print()

# 测试 resolve
print("=== ParamResolver.resolve 测试 ===\n")

print("6. 解析 DynamicBody:")
try:
    result = ParamResolver.resolve(
        func=test_new_style_dynamic_body,
        request=None,
        body_data={'name': 'test', 'age': 18}
    )
    print(f"   解析结果: {result}")
    body = result.get('body')
    if body:
        print(f"   body.name = {body.name}")
        print(f"   body.age = {body.age}")
except Exception as e:
    print(f"   错误: {e}")
print()

print("7. 解析类型化参数:")
try:
    result = ParamResolver.resolve(
        func=test_new_style_typed_params,
        request=None,
        url_params={'id': '123'},
        query_params={'name': 'hello'}
    )
    print(f"   解析结果: {result}")
    print(f"   id 类型: {type(result.get('id'))}")
except Exception as e:
    print(f"   错误: {e}")
print()

print("=== 测试完成 ===")




# -*- coding: utf-8 -*-
"""Cullinan DynamicBody

动态请求体类，支持属性访问方式。

Author: Plumeink
"""

from typing import Any, Dict, Iterator, KeysView, ValuesView, ItemsView, Optional, List, Union


class _Empty:
    """空值哨兵类，用于区分 None 和真正的空"""
    _instance = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __repr__(self):
        return '<EMPTY>'

    def __bool__(self):
        return False


# 空值哨兵
EMPTY = _Empty()


class DynamicBody:
    """动态请求体类

    提供类似 JavaScript 对象的属性访问方式，避免频繁使用 .get()。

    Features:
    - 属性访问: body.name
    - 安全访问: body.get('name', default)
    - 嵌套安全访问: body.get_nested('user.address.city', 'Unknown')
    - 快捷判空: body.has('name'), body.is_empty(), body.is_not_empty()
    - 字典兼容: body['name'], 'name' in body
    - 迭代支持: for key in body
    - 可变性: body.new_field = value
    - 嵌套访问: body.user.name (嵌套字典自动转换)
    - 链式安全访问: body.safe.user.address.city (不抛异常)

    Usage:
        # 推荐语法（避免 non-default parameter 问题）
        @post_api(url="/users")
        async def create_user(
            self,
            auth: str = Header(alias="Authorization"),
            body: DynamicBody = DynamicBody(),  # 使用默认值语法
        ):
            print(body.name, body.age)

        # 简化语法（无其他有默认值的参数时）
        @post_api(url="/users")
        async def create_user(self, body: DynamicBody):
            print(body.name, body.age)

    Example:
        body = DynamicBody({'name': 'test', 'age': 18, 'user': {'id': 1}})

        # 属性访问
        print(body.name)  # 'test'
        print(body.age)   # 18

        # 嵌套访问
        print(body.user.id)  # 1

        # 安全访问
        print(body.get('email', 'default@example.com'))

        # 嵌套安全访问
        print(body.get_nested('user.address.city', 'Unknown'))

        # 快捷判空
        if body.has('name'):
            print('has name')
        if body.is_not_empty():
            print('body has data')

        # 链式安全访问 (不抛异常)
        city = body.safe.user.address.city.value_or('Unknown')

        # 检查存在性
        if 'name' in body:
            print('has name')

        # 设置新字段
        body.email = 'test@example.com'

        # 转换为字典
        data = body.to_dict()
    """

    __slots__ = ('_data',)

    def __init__(self, data: Dict[str, Any] = None):
        """初始化动态请求体

        Args:
            data: 原始数据字典
        """
        object.__setattr__(self, '_data', data if data is not None else {})

    def __getattr__(self, name: str) -> Any:
        """属性访问

        Args:
            name: 属性名

        Returns:
            属性值 (嵌套字典自动转换为 DynamicBody)

        Raises:
            AttributeError: 属性不存在
        """
        data = object.__getattribute__(self, '_data')
        if name in data:
            value = data[name]
            # 嵌套字典自动转换为 DynamicBody
            if isinstance(value, dict):
                return DynamicBody(value)
            return value
        raise AttributeError(f"'{type(self).__name__}' object has no attribute '{name}'")

    def __setattr__(self, name: str, value: Any) -> None:
        """设置属性

        Args:
            name: 属性名
            value: 属性值
        """
        if name == '_data':
            object.__setattr__(self, name, value)
        else:
            self._data[name] = value

    def __delattr__(self, name: str) -> None:
        """删除属性

        Args:
            name: 属性名
        """
        data = object.__getattribute__(self, '_data')
        if name in data:
            del data[name]
        else:
            raise AttributeError(f"'{type(self).__name__}' object has no attribute '{name}'")

    def __getitem__(self, key: str) -> Any:
        """字典式访问

        Args:
            key: 键名

        Returns:
            对应的值
        """
        value = self._data[key]
        if isinstance(value, dict):
            return DynamicBody(value)
        return value

    def __setitem__(self, key: str, value: Any) -> None:
        """字典式设置

        Args:
            key: 键名
            value: 值
        """
        self._data[key] = value

    def __delitem__(self, key: str) -> None:
        """字典式删除

        Args:
            key: 键名
        """
        del self._data[key]

    def __contains__(self, name: str) -> bool:
        """检查键是否存在

        Args:
            name: 键名

        Returns:
            是否存在
        """
        return name in self._data

    def __iter__(self) -> Iterator[str]:
        """迭代键名"""
        return iter(self._data)

    def __len__(self) -> int:
        """返回字段数量"""
        return len(self._data)

    def __bool__(self) -> bool:
        """布尔值判断 (非空为 True)"""
        return bool(self._data)

    def __repr__(self) -> str:
        return f"DynamicBody({self._data!r})"

    def __str__(self) -> str:
        return str(self._data)

    def __eq__(self, other: Any) -> bool:
        if isinstance(other, DynamicBody):
            return self._data == other._data
        if isinstance(other, dict):
            return self._data == other
        return False

    # =========================================================================
    # 字典兼容方法
    # =========================================================================

    def get(self, name: str, default: Any = None) -> Any:
        """安全获取属性值

        Args:
            name: 属性名
            default: 默认值

        Returns:
            属性值或默认值

        Example:
            email = body.get('email', 'default@example.com')
        """
        value = self._data.get(name, default)
        if isinstance(value, dict):
            return DynamicBody(value)
        return value

    def keys(self) -> KeysView[str]:
        """返回所有键"""
        return self._data.keys()

    def values(self) -> ValuesView[Any]:
        """返回所有值"""
        return self._data.values()

    def items(self) -> ItemsView[str, Any]:
        """返回所有键值对"""
        return self._data.items()

    def to_dict(self) -> Dict[str, Any]:
        """转换为普通字典

        Returns:
            字典副本
        """
        return dict(self._data)

    def update(self, data: Dict[str, Any] = None, **kwargs) -> None:
        """更新数据

        Args:
            data: 要合并的字典
            **kwargs: 要设置的键值对
        """
        if data:
            self._data.update(data)
        if kwargs:
            self._data.update(kwargs)

    def pop(self, name: str, *default) -> Any:
        """弹出并返回值

        Args:
            name: 键名
            *default: 默认值 (可选)

        Returns:
            被弹出的值
        """
        return self._data.pop(name, *default)

    def setdefault(self, name: str, default: Any = None) -> Any:
        """设置默认值并返回

        Args:
            name: 键名
            default: 默认值

        Returns:
            现有值或设置的默认值
        """
        return self._data.setdefault(name, default)

    def clear(self) -> None:
        """清空所有数据"""
        self._data.clear()

    # =========================================================================
    # 快捷判空方法
    # =========================================================================

    def has(self, name: str) -> bool:
        """检查属性是否存在

        比 'name' in body 更语义化的方法。

        Args:
            name: 属性名

        Returns:
            属性是否存在

        Example:
            if body.has('email'):
                send_email(body.email)
        """
        return name in self._data

    def has_value(self, name: str) -> bool:
        """检查属性是否存在且值非空（不为 None、空字符串、空列表等）

        Args:
            name: 属性名

        Returns:
            属性是否存在且有值

        Example:
            if body.has_value('email'):
                send_email(body.email)  # 确保 email 不是 None 或 ''
        """
        if name not in self._data:
            return False
        value = self._data[name]
        # None, '', [], {}, 0, False 等都返回 False
        return bool(value)

    def is_empty(self) -> bool:
        """检查是否为空

        Returns:
            如果没有任何数据返回 True

        Example:
            if body.is_empty():
                return {'error': 'No data provided'}
        """
        return len(self._data) == 0

    def is_not_empty(self) -> bool:
        """检查是否非空

        Returns:
            如果有数据返回 True

        Example:
            if body.is_not_empty():
                process(body)
        """
        return len(self._data) > 0

    def is_null(self, name: str) -> bool:
        """检查属性值是否为 None

        Args:
            name: 属性名

        Returns:
            如果属性不存在或值为 None 返回 True

        Example:
            if body.is_null('optional_field'):
                body.optional_field = default_value
        """
        return self._data.get(name) is None

    def is_not_null(self, name: str) -> bool:
        """检查属性值是否不为 None

        Args:
            name: 属性名

        Returns:
            如果属性存在且值不为 None 返回 True

        Example:
            if body.is_not_null('callback_url'):
                notify(body.callback_url)
        """
        return name in self._data and self._data[name] is not None

    # =========================================================================
    # 嵌套安全访问方法
    # =========================================================================

    def get_nested(self, path: str, default: Any = None, separator: str = '.') -> Any:
        """嵌套路径安全访问

        通过点分隔路径安全访问嵌套属性，任意层级不存在都返回默认值。

        Args:
            path: 点分隔的路径，如 'user.address.city'
            default: 默认值
            separator: 路径分隔符，默认 '.'

        Returns:
            嵌套属性值或默认值

        Example:
            city = body.get_nested('user.address.city', 'Unknown')
            # 等价于：
            # city = body.get('user', {}).get('address', {}).get('city', 'Unknown')
            # 但更简洁且安全
        """
        keys = path.split(separator)
        current = self._data

        for key in keys:
            if isinstance(current, dict):
                if key not in current:
                    return default
                current = current[key]
            elif isinstance(current, DynamicBody):
                if key not in current._data:
                    return default
                current = current._data[key]
            else:
                return default

        if isinstance(current, dict):
            return DynamicBody(current)
        return current

    def get_list(self, name: str, default: List = None) -> List:
        """安全获取列表属性

        Args:
            name: 属性名
            default: 默认值，默认为空列表

        Returns:
            列表值或默认值

        Example:
            tags = body.get_list('tags')  # 返回 [] 如果 tags 不存在
            for tag in tags:
                process(tag)
        """
        if default is None:
            default = []
        value = self._data.get(name)
        if value is None:
            return default
        if isinstance(value, list):
            return value
        return default

    def get_str(self, name: str, default: str = '') -> str:
        """安全获取字符串属性

        Args:
            name: 属性名
            default: 默认值，默认为空字符串

        Returns:
            字符串值或默认值

        Example:
            name = body.get_str('name')  # 返回 '' 如果 name 不存在
        """
        value = self._data.get(name)
        if value is None:
            return default
        return str(value)

    def get_int(self, name: str, default: int = 0) -> int:
        """安全获取整数属性

        Args:
            name: 属性名
            default: 默认值，默认为 0

        Returns:
            整数值或默认值

        Example:
            age = body.get_int('age')  # 返回 0 如果 age 不存在或无法转换
        """
        value = self._data.get(name)
        if value is None:
            return default
        try:
            return int(value)
        except (ValueError, TypeError):
            return default

    def get_float(self, name: str, default: float = 0.0) -> float:
        """安全获取浮点数属性

        Args:
            name: 属性名
            default: 默认值，默认为 0.0

        Returns:
            浮点数值或默认值

        Example:
            price = body.get_float('price')  # 返回 0.0 如果 price 不存在
        """
        value = self._data.get(name)
        if value is None:
            return default
        try:
            return float(value)
        except (ValueError, TypeError):
            return default

    def get_bool(self, name: str, default: bool = False) -> bool:
        """安全获取布尔属性

        支持多种布尔值表示：True/False, 'true'/'false', 1/0, 'yes'/'no'

        Args:
            name: 属性名
            default: 默认值，默认为 False

        Returns:
            布尔值或默认值

        Example:
            active = body.get_bool('active')  # 返回 False 如果 active 不存在
        """
        value = self._data.get(name)
        if value is None:
            return default
        if isinstance(value, bool):
            return value
        if isinstance(value, str):
            return value.lower() in ('true', '1', 'yes', 'on')
        if isinstance(value, (int, float)):
            return bool(value)
        return default

    # =========================================================================
    # 链式安全访问器
    # =========================================================================

    @property
    def safe(self) -> 'SafeAccessor':
        """返回安全访问器，支持链式访问而不抛异常

        Returns:
            SafeAccessor 实例

        Example:
            # 传统方式可能抛异常
            # city = body.user.address.city  # AttributeError if any level is missing

            # 使用安全访问器
            city = body.safe.user.address.city.value_or('Unknown')

            # 或者检查是否存在
            if body.safe.user.address.city.exists:
                print(body.safe.user.address.city.value)
        """
        return SafeAccessor(self._data)

    def copy(self) -> 'DynamicBody':
        """返回浅拷贝

        Returns:
            新的 DynamicBody 实例
        """
        return DynamicBody(self._data.copy())


class SafeAccessor:
    """安全访问器

    支持链式访问而不抛出 AttributeError，任意层级不存在都安全处理。

    Example:
        body = DynamicBody({'user': {'name': 'John'}})

        # 安全访问存在的属性
        name = body.safe.user.name.value  # 'John'

        # 安全访问不存在的属性
        city = body.safe.user.address.city.value_or('Unknown')  # 'Unknown'

        # 检查是否存在
        if body.safe.user.email.exists:
            send_email(body.safe.user.email.value)
    """

    __slots__ = ('_data', '_exists')

    def __init__(self, data: Any, exists: bool = True):
        """初始化安全访问器

        Args:
            data: 当前数据
            exists: 数据是否存在
        """
        object.__setattr__(self, '_data', data)
        object.__setattr__(self, '_exists', exists)

    def __getattr__(self, name: str) -> 'SafeAccessor':
        """安全获取属性，不抛异常

        Args:
            name: 属性名

        Returns:
            新的 SafeAccessor 实例
        """
        data = object.__getattribute__(self, '_data')
        exists = object.__getattribute__(self, '_exists')

        if not exists:
            return SafeAccessor(None, False)

        if isinstance(data, dict):
            if name in data:
                return SafeAccessor(data[name], True)
            return SafeAccessor(None, False)

        if isinstance(data, DynamicBody):
            if name in data._data:
                return SafeAccessor(data._data[name], True)
            return SafeAccessor(None, False)

        return SafeAccessor(None, False)

    @property
    def value(self) -> Any:
        """获取当前值

        Returns:
            当前值，如果不存在返回 None
        """
        if not self._exists:
            return None
        if isinstance(self._data, dict):
            return DynamicBody(self._data)
        return self._data

    def value_or(self, default: Any) -> Any:
        """获取值或默认值

        Args:
            default: 默认值

        Returns:
            当前值或默认值
        """
        if not self._exists or self._data is None:
            return default
        if isinstance(self._data, dict):
            return DynamicBody(self._data)
        return self._data

    @property
    def exists(self) -> bool:
        """检查值是否存在

        Returns:
            是否存在
        """
        return self._exists

    @property
    def is_null(self) -> bool:
        """检查值是否为 None

        Returns:
            是否为 None
        """
        return not self._exists or self._data is None

    @property
    def is_not_null(self) -> bool:
        """检查值是否不为 None

        Returns:
            是否不为 None
        """
        return self._exists and self._data is not None

    def __bool__(self) -> bool:
        """布尔值判断"""
        return self._exists and bool(self._data)

    def __repr__(self) -> str:
        if self._exists:
            return f"SafeAccessor({self._data!r})"
        return "SafeAccessor(<missing>)"
